function [Corr,CRseed] = CalcCorrfromLW(srcpar,mech,acf,CRseed);
%
%  function [Corr,CRseed] = CalcCorrfromLW(srcpar,'mech','acf',CRseed)
%  calculates the correlation length or fractal dimension
%  (i.e. spectral decay) of the power spectrum ACF for
%  a given moment. The final value is picked from a normal 
%  distribution within the error bounds of measured correlation
%  length and fractal dimensions.
%
%  input: srcpar - [W L M] for fault length, width, magnitude
%	  mech 	- 'ss' for strike-slip events; 
%		  'ds' for dip-slip events
%	  acf 	- string to denote autocorrelation function
%		  'gs' for Gaussian
%		  'ex' for exponential
%		  'ak' for anisotropic von Karman
%		  'fr' for the fractal case
%	  CRseed- optional array of seed-values;
%		  size 1x2 for 'fr'
%		  size 4x2 for 'gs','ex',
%  		  size 5x2 for 'ak'
%		  if given as 'off', the sample is given as computed,
%		  without randomization (ignoring aleatory variability)
%
%  output: Corr	- vector of correlation length Cz, Cx
%		  for anisotropic von Karman: Corr = [Cz Cx H]
%	          for Gauss or exponetial: Corr = [Cz Cx]
%		  for fractal case: Corr = D;

%  Written by Martin Mai (mmai@pangea.Stanford.EDU) 
%  07/06/98
%  last change 03/01/2000 - saving seed-values added
%  ------------------------------------------------

if nargin <= 3, CRseed = 'off'; end

%%% check source parameter input
if length(srcpar) == 1
   error(' CalcCorrfromLW stopped: not enough srcpar-input values given');
elseif length(srcpar) == 2
   W = srcpar(1); 
   L = srcpar(2);
   disp('magnitude computed from source-scaling relations');
   [L,W,D,M,RWCseed] = CalcDimDMWC(L,W,mech,'off');
   disp(sprintf('M = %.2f',M));
else
   W = srcpar(1); 
   L = srcpar(2);
   M = srcpar(3);
end


%%% constants calculated from lsqr-fits of measured correlation length

if ((acf == 'gs') | (acf == 'GS'))
   disp('++ Coefficients for corr. lengths of Gaussian ACF ++')
   disp('++ are fixed to increase slowly with EQ-magnitude ++');
   
        bxM = -1.500;   mxM = 0.300;	varxM = 0.1;
      	bxL =  3.000; 	mxL = 0.015; 	varxL = 0.4;	
	bxW =  3.300; 	mxW = 0.015; 	varxW = 0.4;
        bzM = -1.500;	mzM = 0.300;	varzM = 0.1;	
      	bzL =  2.900; 	mzL = 0.010; 	varzL = 0.4;			
      	bzW =  3.000; 	mzW = 0.015; 	varzW = 0.4;


elseif ((acf == 'ex') | (acf == 'EX'))

   if ((mech == 'ss') | (mech == 'SS'));
      	bxM = -3.163;   mxM = 0.625;	varxM = 0.14;
	bxL =  0.737; 	mxL = 0.392; 	varxL = 2.24;			
      	bxW = -4.949; 	mxW = 1.815; 	varxW = 6.01;	
	bzM = -1.600;	mzM = 0.330;	varzM = 0.12;
	bzL =  2.422; 	mzL = 0.063; 	varzL = 1.08;	
      	bzW = -0.035; 	mzW = 0.434; 	varzW = 0.27;

   elseif ((mech == 'ds') | (mech == 'DS')); 
      	bxM = -2.571;   mxM = 0.522;	varxM = 0.15;
      	bxL = -0.210; 	mxL = 0.430; 	varxL = 1.68;		  	
      	bxW =  0.499; 	mxW = 0.629; 	varxW = 4.51;
	bzM = -1.851;	mzM = 0.394;	varzM = 0.15;	
	bzL =  1.211; 	mzL = 0.241; 	varzL = 3.16;	
      	bzW =  0.646; 	mzW = 0.397; 	varzW = 0.72;

   elseif ((mech == 'al') | (mech == 'AL')); 
      	bxM = -2.811;   mxM = 0.565;	varxM = 0.15;
      	bxL =  0.079; 	mxL = 0.415; 	varxL = 2.00;	
	bxW =  4.865; 	mxW = 0.591; 	varxW = 7.34;
	bzM = -1.832;	mzM = 0.378;	varzM = 0.18;	
      	bzL =  1.095; 	mzL = 0.168; 	varzL = 4.16;			
      	bzW =  0.427; 	mzW = 0.403; 	varzW = 0.56;
   end

elseif ((acf == 'ak') | (acf == 'AK'))
	
   meanH = 0.77; varH = 0.23;

   if ((mech == 'ss') | (mech == 'SS'));
      	bxM = -2.928;   mxM = 0.588;	varxM = 0.19;
      	bxL =  1.855;	mxL = 0.341; 	varxL = 2.79;			
      	bxW = -4.870;	mxW = 1.741; 	varxW = 4.61;
	bzM = -1.619;   mzM = 0.328;	varzM = 0.15;
	bzL =  2.247;	mzL = 0.059; 	varzL = 1.41;	
      	bzW = -0.386;	mzW = 0.438; 	varzW = 0.75;

   elseif ((mech == 'ds') | (mech == 'DS')); 
      	bxM = -2.433;   mxM = 0.492;	varxM = 0.14;
      	bxL =  1.096; 	mxL = 0.314; 	varxL = 1.26;		  	
      	bxW =  1.832; 	mxW = 0.449; 	varxW = 3.87;	
	bzM = -1.785;   mzM = 0.377;	varzM = 0.17;
	bzL =  1.118; 	mzL = 0.208; 	varzL = 2.87;	
      	bzW =  0.584; 	mzW = 0.347; 	varzW = 0.76;

   elseif ((mech == 'al') | (mech == 'AL')); 
       	bxM = -2.595;   mxM = 0.527;	varxM = 0.19;
     	bxL =  1.541; 	mxL = 0.326; 	varxL = 2.31;	
	bxW =  6.072; 	mxW = 0.416; 	varxW = 6.80;
	bzM = -1.801;   mzM = 0.367;	varzM = 0.18;
      	bzL =  1.074; 	mzL = 0.147; 	varzL = 3.61;			
      	bzW =  0.552; 	mzW = 0.349; 	varzW = 0.78;
   end

elseif ((acf == 'fr') | (acf == 'FR'));
   meanD = 2.29; varD = 0.23;
end

%%% obtain fractal dimensions and/or correlation length 
%%% by successive call to function NORMSAMPLE

%%% compute fractal dimension in case of fractal slip
if ((acf == 'fr') | (acf == 'FR'))
   kc = 2*pi/(sqrt(L*W));	% corner wavenumber
   if isempty(CRseed) == 1
     CRseed = zeros(1,2);
     [D,CRseed(1,:)] = NormSample(meanD,0.5*varD,100);

   elseif isempty(CRseed) ~= 1 
     if isstr(CRseed) == 1
	if CRseed == 'off'
	   D = meanD;
	else, error('no valid CRseed-value in CalcCorrfromLW');
	end
     else
       [D,CRseed(1,:)] = NormSample(meanD,0.5*varD,100,CRseed);
     end
   end

%%% compute correlation lengths

else
    CxM = bxM + mxM*M;
    CxL = bxL + mxL*L;	
    CxW = bxW + mxW*W;
    CzM = bzM + mzM*M;
    CzL = bzL + mzL*L;	
    CzW = bzW + mzL*W;

        if isempty(CRseed) == 1
	   if (acf == 'ak') | (acf == 'AK')
              CRseed = zeros(7,2);
           else
              CRseed = zeros(6,2);
           end

	   %%% along-strike direction, from length and width
	   [dxM,CRseed(1,:)] = NormSample(CxM,0.5*varxM,100);
	   [dxL,CRseed(2,:)] = NormSample(CxL,0.5*varxL,100);
	   [dxW,CRseed(3,:)] = NormSample(CxW,0.5*varxW,100);
	
	   %%% down-dip direction, from length and width
 	   [dzM,CRseed(4,:)] = NormSample(CzM,0.5*varzM,100);
	   [dzL,CRseed(5,:)] = NormSample(CzL,0.5*varzL,100);
	   [dzW,CRseed(6,:)] = NormSample(CzW,0.5*varzW,100);

        elseif isempty(CRseed) ~= 1
	
	  if isstr(CRseed) == 1
	    if CRseed == 'off'
	      dxL = CxL;  dxW = CxW;  dxM = CxM;
 	      dzL = CzL;  dzW = CzW;  dzM = CzM;
 	    else; error('no valid CRseed-value in CalcCorrfromLW');
	    end
	  
          else

	    %%% along-strike direction, from length and width
	    [dxM,CRseed(1,:)] = NormSample(CxM,0.5*varxM,100,CRseed(1,:));
 	    [dxL,CRseed(2,:)] = NormSample(CxL,0.5*varxL,100,CRseed(2,:));
	    [dxW,CRseed(3,:)] = NormSample(CxW,0.5*varxW,100,CRseed(3,:));

	    %%% down-dip direction, from length and width
	    [dzM,CRseed(4,:)] = NormSample(CzM,0.5*varzM,100,CRseed(4,:));
	    [dzL,CRseed(5,:)] = NormSample(CzL,0.5*varzL,100,CRseed(5,:));
	    [dzW,CRseed(6,:)] = NormSample(CzW,0.5*varzW,100,CRseed(6,:));
          end

	end
	
	%%% use mean values for the correlation length, as obtained
	%%% from parametrization on fault length and fault width
	dxM = 10.^dxM; dzM = 10.^dzM;
	Cx = mean([dxM dxL dxW]); 
	Cz = mean([dzM dzL dzW]); 

	%%% compute Hurstnumber H for anisotropic von Karman
	if ((acf == 'ak') | (acf == 'AK'))
	if isstr(CRseed) == 1
	  if CRseed == 'off'
	 	H = meanH;
 	  else error('no valid CRseed-value in CalcCorrfromLW');
	  end
	else 
	  if CRseed(7,:) == zeros(1,2)
	    [H,CRseed(7,:)] = NormSample(meanH,0.5*varH,100);
          else
	    [H,CRseed(7,:)] = NormSample(meanH,0.5*varH,100,CRseed(7,:));
	  end
	end
	end
end

%%% asemble final output vecor
if (acf == 'gs') | (acf == 'ex') | (acf == 'GS') | (acf == 'EX')
	Corr = [Cz Cx];
elseif ((acf == 'ak') | (acf == 'AK'));
	Corr = [Cz Cx H];
elseif ((acf == 'fr') | (acf == 'FR'));
	Corr = [D kc];
end
