function [Corr,CRseed] = CalcCorrfromLW(L,W,mech,acf,CRseed);
%
%  function [Corr,CRseed] = CalcCorrfromLW(L,W,,'mech','acf',CRseed)
%  calculates the correlation length or fractal dimension
%  (i.e. spectral decay) of the power spectrum ACF for
%  a given moment. The final value is picked from a normal 
%  distribution within the error bounds of measured correlation
%  length and fractal dimensions.
%
%  input: L,W  	- fault length and width, [in km]
%	  mech 	- 'ss' for strike-slip events; 
%		  'ds' for dip-slip events
%	  acf 	- string to denote autocorrelation function
%		  'gs' for Gaussian
%		  'ex' for exponential
%		  'ak' for anisotropic von Karman
%		  'fr' for the fractal case
%	  CRseed- optional array of seed-values;
%		  size 1x2 for 'fr'
%		  size 4x2 for 'gs','ex',
%		  size 5x2 for 'ak'
%
%  output: Corr	- vector of correlation length Cz, Cx
%		  for anisotropic von Karman: Corr = [Cz Cx H]
%	          for Gauss or exponetial: Corr = [Cz Cx]
%		  for fractal case: Corr = D;

%  Written by Martin Mai (mmai@pangea.Stanford.EDU) 
%  07/06/98
%  last change 03/01/2000 - saving seed-values added
%  ------------------------------------------------

if nargin <= 4, CRseed = []; end

%%% constants calculated from lsqr-fits of measured correlation length

if ((acf == 'gs') | (acf == 'GS'))

   if ((mech == 'ss') | (mech == 'SS'));
      	bxL = 2.782; 	mxL = 0.020; 	varxL = 0.347;			
      	bxW = 2.722; 	mxW = 0.064; 	varxW = 0.535;	
	bzL = 3.011; 	mzL = 0.003; 	varzL = 0.329;	
      	bzW = 2.378; 	mzW = 0.054; 	varzW = 0.247;

   elseif ((mech == 'ds') | (mech == 'DS')); 
      	bxL =  3.146; 	mxL = 0.012; 	varxL = 0.534;		  	
      	bxW =  3.242; 	mxW = 0.015; 	varxW = 0.624;	
	bzL =  2.988; 	mzL = 0.010; 	varzL = 0.478;	
      	bzW =  3.021; 	mzW = 0.015; 	varzW = 0.501;

   elseif ((mech == 'al') | (mech == 'AL')); 
      	bxL =  3.063; 	mxL = 0.014; 	varxL = 0.448;	
	bxW =  3.354; 	mxW = 0.015; 	varxW = 0.586;	
      	bzL =  2.898; 	mzL = 0.009; 	varzL = 0.433;			
      	bzW =  2.961; 	mzW = 0.016; 	varzW = 0.396;
   end

elseif ((acf == 'ex') | (acf == 'EX'))

   if ((mech == 'ss') | (mech == 'SS'));
      	bxL = -0.265; 	mxL = 0.261; 	varxL = 3.293;			
      	bxW =  3.183; 	mxW = 0.549; 	varxW = 6.849;	
	bzL =  2.460; 	mzL = 0.048; 	varzL = 1.255;	
      	bzW =  0.069; 	mzW = 0.309; 	varzW = 1.092;

   elseif ((mech == 'ds') | (mech == 'DS')); 
      	bxL = -0.982; 	mxL = 0.331; 	varxL = 3.334;		  	
      	bxW =  0.241; 	mxW = 0.461; 	varxW = 6.354;	
	bzL =  0.163; 	mzL = 0.194; 	varzL = 3.572;	
      	bzW = -0.102; 	mzW = 0.301; 	varzW = 0.828;

   elseif ((mech == 'al') | (mech == 'AL')); 
      	bxL = -1.631; 	mxL = 0.319; 	varxL = 3.592;	
	bxW =  3.065; 	mxW = 0.432; 	varxW = 6.711;	
      	bzL = -0.757; 	mzL = 0.168; 	varzL = 4.030;			
      	bzW =  0.090; 	mzW = 0.300; 	varzW = 0.951;
   end

elseif ((acf == 'ak') | (acf == 'AK'))
	
   meanH = 0.8690; varH = 0.3991;

   if ((mech == 'ss') | (mech == 'SS'));
      	bxL =  0.789;	mxL = 0.226; 	varxL = 3.876;			
      	bxW =  4.322;	mxW = 0.436; 	varxW = 6.523;	
	bzL =  1.016;	mzL = 0.066; 	varzL = 1.404;	
      	bzW = -0.540;	mzW = 0.306; 	varzW = 1.694;

   elseif ((mech == 'ds') | (mech == 'DS')); 
      	bxL = -0.228; 	mxL = 0.262; 	varxL = 2.843;		  	
      	bxW =  0.632; 	mxW = 0.367; 	varxW = 4.825;	
	bzL = -0.411; 	mzL = 0.161; 	varzL = 3.537;	
      	bzW = -0.786; 	mzW = 0.255; 	varzW = 1.067;

   elseif ((mech == 'al') | (mech == 'AL')); 
      	bxL = -0.208; 	mxL = 0.255; 	varxL = 3.358;	
	bxW =  3.773; 	mxW = 0.335; 	varxW = 6.017;	
      	bzL = -1.048; 	mzL = 0.144; 	varzL = 3.286;			
      	bzW = -0.138; 	mzW = 0.249; 	varzW = 1.467;
   end

elseif ((acf == 'fr') | (acf == 'FR'));
   meanD = 2.2132; varD = 0.3933;
end

%%% obtain fractal dimensions and/or correlation length 
%%% by successive call to function NORMSAMPLE

%%% compute fractal dimension in case of fractal slip
if ((acf == 'fr') | (acf == 'FR'))
   kc = 2*pi/(sqrt(L*W));	% corner wavenumber
   if isempty(CRseed) == 1
     CRseed = zeros(1,2);
     [D,CRseed(1,:)] = NormSample(meanD,0.5*varD,100);
   else
     [D,CRseed(1,:)] = NormSample(meanD,0.5*varD,100,CRseed);
   end

%%% compute correlation lengths

else
    CxL = bxL + mxL*L;	CxW = bxW + mxW*W;
    CzL = bzL + mzL*L;	CzW = bzW + mzL*W;

        if isempty(CRseed) == 1
	 if (acf == 'ak') | (acf == 'AK')
           CRseed = zeros(5,2);
         else
           CRseed = zeros(4,2);
         end

	  %%% along-strike direction, from length and width
	  [dxL,CRseed(1,:)] = NormSample(CxL,0.5*varxL,100);
	  [dxW,CRseed(2,:)] = NormSample(CxW,0.5*varxW,100);
	
	   %%% down-dip direction, from length and width
	  [dzL,CRseed(3,:)] = NormSample(CzL,0.5*varzL,100);
	  [dzW,CRseed(4,:)] = NormSample(CzW,0.5*varzW,100);
 
	else

	  %%% along-strike direction, from length and width
 	  [dxL,CRseed(1,:)] = NormSample(CxL,0.5*varxL,100,CRseed(1,:));
	  [dxW,CRseed(2,:)] = NormSample(CxW,0.5*varxW,100,CRseed(2,:));

	  %%% down-dip direction, from length and width
	  [dzL,CRseed(3,:)] = NormSample(CzL,0.5*varzL,100,CRseed(3,:));
	  [dzW,CRseed(4,:)] = NormSample(CzW,0.5*varzW,100,CRseed(4,:));

	end
	
	%%% use mean values for the correlation length, as obtained
	%%% from parametrization on fault length and fault width
	Cx = mean([dxL dxW]);
	Cz = mean([dzL dzW]);

	%%% compute Hurstnumber H for anisotropic von Karman
	if ((acf == 'ak') | (acf == 'AK'))
	  if CRseed(5,:) == zeros(1,2)
	    [H,CRseed(5,:)] = NormSample(meanH,0.5*varH,100);
	  else
	    [H,CRseed(5,:)] = NormSample(meanH,0.5*varH,100,CRseed(5,:));
	  end
	end
end

%%% asemble final output vecor
if (acf == 'gs') | (acf == 'ex') | (acf == 'GS') | (acf == 'EX')
	Corr = [Cz Cx];
elseif ((acf == 'ak') | (acf == 'AK'));
	Corr = [Cz Cx H];
elseif ((acf == 'fr') | (acf == 'FR'));
	Corr = [D kc];
end
