function [L,W,D,Mw,RWCseed] = CalcDimWC(L,W,mech,RWCseed);
%
%  function [L,W,D,Mw,RWCseed] = CalcDimWC(L,W,'mech',RWCseed) 
%  estimates the mean slip and seismic moment (moment
%  magnitude) for given fault length and width using the
%  empirical relation of Wells & Coppersmith.
%  The source parameters are randomly picked from a 
%  normal distribution around the computed values
%
%  input: L,W 	 - length and width, in km
%	  mech 	 - 'ss' or 'SS' for strike-slip
%		   'ds' or 'DS' for dip-slip
%		   'al' or 'AL' for all events
%	  RWCseed- optional 3x2-array for seed-values
%
%  output: [L,W,D,Mw] - fault length and width (in km)
%			mean slip (in m),resulting moment magnitude
%	  RWCseed-  3x2-array of seed-values for source realizations

%  Written by Martin Mai (mmai@pangea.Stanford.EDU) 
%  09/21/99
%  ------------------------------------------------

if nargin <= 3; 
	RWCseed = [];
end;

mu = 3.3*1e10;				% rigidity;

%%% define scaling constants, as published in Wells & Coppersmith (1994)

  if ((mech == 'ss' ) | (mech == 'SS'))  ;
	mL = 1.49;	bL = 4.33; 	varL = 0.24;
      	mW = 2.59; 	bW = 3.80;	varW = 0.45;
     	mA = 1.02; 	bA = 3.98; 	varA = 0.23;
  elseif ((mech == 'ds' ) | (mech == 'DS'))  ;
      	mL = 1.49; 	bL = 4.49; 	varL = 0.26;
      	mW = 1.95; 	bW = 4.37; 	varW = 0.32;
     	mA = 0.90; 	bA = 4.33;  	varA = 0.25;  
  elseif ((mech == 'al' ) | (mech == 'AL'))  ;
      	mL = 1.49; 	bL = 4.38;  	varL = 0.26;
      	mW = 2.25; 	bW = 4.06; 	varW = 0.41;
     	mA = 0.98; 	bA = 4.07; 	varA = 0.24;  
end



%%% compute "central" values of length, width, area, mean slip
Mlo = bL + mL*log10(L);
Mwo = bW + mW*log10(W);
Mao = bA + mA*log10(L*W);


%%% call function NORMSAMPLE to find values for the moments estimated
%%% from the given fault length, and area from normal distributions; 
if isempty(RWCseed) == 1
  RWCseed = zeros(3,2);
  [Ml,RWCseed(1,:)] = NormSample(Mlo,0.5*varL,100);
  [Mw,RWCseed(2,:)] = NormSample(Mwo,0.5*varW,100);
  [Ma,RWCseed(3,:)] = NormSample(Mao,0.5*varA,100);
else
  [Ml,RWCseed(1,:)] = NormSample(Mlo,0.5*varL,100,RWCseed(1,:));
  [Mw,RWCseed(2,:)] = NormSample(Mwo,0.5*varW,100,RWCseed(2,:));
  [Ma,RWCseed(3,:)] = NormSample(Mao,0.5*varA,100,RWCseed(3,:));
end


%%% use median value of the three estimates
Mw = median([Ml Mw Ma]);


%%% calculate mean slip [in m] for chosen Mw and 
%%% realization of fault dimensions 
Mo = fMw2MoN(Mw); 
D = (Mo/(mu*L*1e3*W*1e3));
