function [L,W,D,Mw,Rseed] = CalcDimfromA(A,mech,rel,Rseed);
%
%  function [L,W,D,Mw,Rseed] = CalcDimfromA(A,'mech',rel,Rseed) 
%  estimates the seismic moment (moment magnitude) for a given 
%  fault area, using empirical relation by either Wells & 
%  Coppersmith (1994), Mai & Beroza (2000) or the USGS 
%  WorkingGroup99 (2000).
%  The source parameters are randomly picked from a 
%  normal distribution around the computed values
%
%  input: A 	- area or [W L] length and width, in km
%	  mech 	- 'ss' or 'SS' for strike-slip
%		  'ds' or 'DS' for dip-slip
%		  'al' or 'AL' for all events
%	  rel	- 'MB' to use Mai & Beroza (2000) coefficients
%		  'WC' to use Wells & Coppersmith (1994) coefficients
%		  'WG' to use WorkingGroup99 (2000) coefficients
%	  Rseed - optional 3x2-array for seed-values
%		  if given as 'off', the sample is given as computed,
%		  without randomization (i.e. ignoring aleatory variability)
%
%  output:[L,W,D,Mw] - fault length and width (in km)
%		       mean slip (in m) resulting moment magnitude
%	  Rseed - 3x2-array of seed-values for source realizations

%  Written by Martin Mai (mmai@pangea.Stanford.EDU) 
%  05/14/2001
%  ------------------------------------------------

if nargin == 2; 
	rel = 'MB'; Rseed = 'off';
elseif nargin == 3;
	Rseed = 'off';
end;

mu = 3.3*1e10;				% rigidity;

%%% get area if length, width is given
if length(A) == 2; 
   W = A(1); L = A(2); area = L*W ; 
else 
   area = A;
end

%%% define scaling constants
%%% as published in Wells & Coppersmith (1994)
if rel == 'WC'
  if ((mech == 'al' ) | (mech == 'AL'))  ;
      	mW = 0.32; 	bW = -1.01; 	varW = 0.15;
	mA = 0.98; 	bA =  4.07; 	varA = 0.24; 
  elseif ((mech == 'ss' ) | (mech == 'SS'))  ;
        mW = 0.27;      bW = -0.76;     varW = 0.14;
        mA = 1.02; 	bA =  3.98; 	varA = 0.23;
  elseif ((mech == 'ds' ) | (mech == 'DS'))  ;
      	mW = 0.41; 	bW = -1.61; 	varW = 0.15;
	mA = 0.90; 	bA =  4.33;  	varA = 0.25;  
  end

elseif rel == 'WG'
	disp(' ++ Did you remember that the WG99 relation is developed ++');
	disp(' ++    for large Californian strike-slip earthquakes?    ++');
	mW = 0.27;      bW = -0.76;     varW = 0.14;
	mA = 1.00; 	bA =  4.20; 	varA = 0.12;

elseif rel == 'MB'
   source = {'eff' 'inv' 'eff'};	
     	% the philosophy behind this variable 'source' is that the
	% "effective" dimensions may lead to small source size (hence 
	% large stress drop) while the the dimensions from source 
	% inversions may overpredict the actual earthquake size; 
	% we therefore use a weighted average. {'eff' 'eff'} or {'inv' 'inv'} 
	% yield the dimenions for each case individually
			
  for i = 1:length(source)
  if source{i} == 'inv'  
    if ((mech == 'al' ) | (mech == 'AL')) 
        mW = 0.292; 	bW = -4.281;	varW = 0.166;
     	mA = 0.96; 	bA =  4.03; 	varA = 0.22;
    elseif ((mech == 'ss' ) | (mech == 'SS'))  ;
        mW = 0.088; 	bW = -0.540; 	varW = 0.117;
     	mA = 1.18; 	bA =  3.41; 	varA = 0.30;
    elseif ((mech == 'ds' ) | (mech == 'DS'))  ;
        mW = 0.329; 	bW = -4.927; 	varW = 0.118;
     	mA = 0.87; 	bA =  4.28;  	varA = 0.23; 
    end

  %%% coefficients derived from effective source dimensions
  elseif source{i} == 'eff'
    if ((mech == 'al' ) | (mech == 'AL'))  ;
        mW = 0.323; 	bW = -5.054;	varW = 0.161;
	mA = 0.84; 	bA =  4.55; 	varA = 0.17;
    elseif ((mech == 'ss' ) | (mech == 'SS'))  ;
        mW = 0.167; 	bW = -2.183; 	varW = 0.112;
     	mA = 0.98; 	bA =  4.21; 	varA = 0.25;
    elseif ((mech == 'ds' ) | (mech == 'DS'))  ;
        mW = 0.350; 	bW = -5.507; 	varW = 0.148;
        mA = 0.77; 	bA =  4.71;  	varA = 0.23; 
    end
  end      
     bw(i) = bW;
     mw(i) = mW;   
     ba(i) = bA;
     ma(i) = mA;
 end 
end


%%% compute "central" value of moment magnitude and width (if not given)
if rel == 'WC' | rel == 'WG'
   Ma = bA + mA*log10(area);
   
   if length(A) == 1
	Wa = bW + mW*Ma;
   else 
	Wa = W;
   end

elseif rel == 'MB'
    for i = 1:length(source)
       Ms(i) = ba(i) + ma(i)*log10(area);
       
       if length(A) == 1
          Ws(i) = mw(i)*(1.5*Ms(i)+ 9.05) + bw(i);
       else
	  Ws = W;
       end
    end

    Ma = mean(Ms);
    Wa = mean(Ws);
end

%%% call function NORMSAMPLE to find values for the moments estimated
%%% from the given fault length, and area from normal distributions; 
if isempty(Rseed) == 1
     Rseed = zeros(1,2);
     [Mw,Rseed] = NormSample(Ma,varA,100);
elseif isempty(Rseed) ~=1 
     if isstr(Rseed) == 1;
        if Rseed == 'off'
          Mw = Ma;
        else error('No valid Rseed-value in CalcDimfromA'); 
        end
     end
else
     [Mw,Rseed] = NormSample(Ma,varA,100,Rseed);
end


%%% compute L if not given explicitely
if length(A) == 1
   W = 10.^Wa;
   L = area/W;
end

%%% calculate mean slip [in m] for chosen Mw and 
%%% realization of fault dimensions 
Mo = fMw2MoN(Mw); 
D = (Mo/(mu*L*1e3*W*1e3));
