function [L,W,D,Mw,RCseed] = CalcDimfromM(Mw,mech,dscale,rel,RCseed);
%
% [L,W,D,Mw,RCseed] = CalcDimfromM(Mw,'mech','dscale',rel,RCseed) 
% computes the rupture dimensions given moment magnitude Mw and the 
% fault mechanism 'mech'. Data are based on scaling relations 
% established in Mai & Beroza (2000) or Wells & Coppersmith (1994).
% The source parameters are randomly picked within 95% confidence
% intervals of a normal distribution around the computed values.
%
%  input: Mw  	- moment magnitude
%	  mech 	- 'ss' for strike-slip
%		  'ds' for dip-slip
%		  'al' for all events
%	  dscale- 'd'  to scale mean slip such that target 
%		       moment is reached exactly (default)
%		  'm'  does not scale mean slip, and actual
%		       moment may differ from target moment
%	  rel	- 'MB' to use Mai & Beroza (2000) coefficients
%		  'WC' to use Wells & Coppersmith (1994) coefficients
%
%	  RCseed- optional 3x2-array for seed-values;
%		  if given as 'off', the sample is given as computed,
%		  without randomization (i.e. ignoring aleatory variability)
%
%  output: [L,W,D,Mw1] - fault length and width (in km)
%			 mean slip (in m) resulting moment magnitude
%	  RCseed-  3x2-array of seed-values for source realizations

%  Written by Martin Mai (mmai@pangea.Stanford.EDU) 
%  06/26/97
%  last change 08/28/2000
%  ------------------------------------------------

if nargin == 2; 
	dscale = 'd'; rel = 'MB'; RCseed = 'off';
elseif nargin == 3;
	rel = 'MB'; RCseed = 'off';
elseif nargin == 4;
	RCseed = 'off';
end;

mu = 3.3*1e10;		% rigidity;
source = {'eff' 'inv' 'inv'};	
source = {'eff' 'eff' 'eff'};
			% to use 'MB'-relations based on source dimension
			% NOTE that the "effective" dimensions may lead to 
			% small source size (hence large stress drop) while the
			% the dimensions from source inversions may overpredict 
			% the actual earthquake size; we therefore use a 
			% weighted average. {'eff' 'eff'} or {'inv' 'inv'} 
			% yield the dimenions for each case individually

%%% define scaling constants, as published in Mai & Beroza (2000).
if rel == 'MB'

   for i = 1:length(source)

       	%%% coefficients from dimensions used in source inversion
       	if source{i} == 'inv'  
	   if ((mech == 'al' ) | (mech == 'AL'))  ;
		mL = 0.354;	bL = -5.204; 	varL = 0.161;
      		mW = 0.292; 	bW = -4.281;	varW = 0.166;
     		mA = 0.644; 	bA = -9.444; 	varA = 0.249;
      		mD = 0.351;  	bD = -4.982; 	varD = 0.243;
 	   elseif ((mech == 'ss' ) | (mech == 'SS'))  ;
      		mL = 0.365; 	bL = -5.149; 	varL = 0.114;
      		mW = 0.088; 	bW = -0.540; 	varW = 0.117;
     		mA = 0.439; 	bA = -5.594;  	varA = 0.213;  
      		mD = 0.549; 	bD = -8.675; 	varD = 0.199;
  	   elseif ((mech == 'ds' ) | (mech == 'DS'))  ;
      		mL = 0.378; 	bL = -5.713;  	varL = 0.178;
      		mW = 0.329; 	bW = -4.927; 	varW = 0.118;
     		mA = 0.707; 	bA = -10.640; 	varA = 0.255;  
      		mD = 0.294; 	bD = -3.880; 	varD = 0.255;   
  	   end

	%%% coefficients from effective source dimensions
 	elseif source{i} == 'eff'
  	   if ((mech == 'al' ) | (mech == 'AL'))  ;
		mL = 0.392;	bL = -6.127; 	varL = 0.160;
      		mW = 0.323; 	bW = -5.054;	varW = 0.161;
     		mA = 0.715; 	bA = -11.181; 	varA = 0.257;
      		mD = 0.285;  	bD = -3.338; 	varD = 0.257;
  	   elseif ((mech == 'ss' ) | (mech == 'SS'))  ;
      		mL = 0.404; 	bL = -6.310; 	varL = 0.118;
      		mW = 0.167; 	bW = -2.183; 	varW = 0.112;
     		mA = 0.571; 	bA = -8.492;  	varA = 0.189;  
      		mD = 0.429; 	bD = -6.026; 	varD = 0.189;
  	   elseif ((mech == 'ds' ) | (mech == 'DS'))  ;
      		mL = 0.403; 	bL = -6.388;  	varL = 0.190;
      		mW = 0.350; 	bW = -5.507; 	varW = 0.148;
     		mA = 0.753; 	bA = -11.896; 	varA = 0.305;  
      		mD = 0.247; 	bD = -2.623; 	varD = 0.305;      
	   end

        end		%% condition on source-dimension type

	%%% compute mean values of length, width, area, mean slip
	Lo(i) = mL*(1.5*Mw + 9.05) + bL;
	Wo(i) = mW*(1.5*Mw + 9.05) + bW;
	Ao(i) = mA*(1.5*Mw + 9.05) + bA;
	Do(i) = mD*(1.5*Mw + 9.05) + bD;

   end			%% loop over number in 'source'

	Lo = mean(Lo); Wo = mean(Wo);
	Ao = mean(Ao); Do = mean(Do);

%%% Wells & Coppersmith coefficients; note that these coefficients 
%%% are given w.r.t. to moment magnitude Mw, as in the publications,
%%% in km (L, W) and m (D), respectively
elseif rel == 'WC'
  if ((mech == 'al' ) | (mech == 'AL'));
	    mL = 0.59;	bL = -2.44; 	varL = 0.16;
      	mW = 0.32; 	bW = -1.01;	    varW = 0.15;
     	mA = 0.91; 	bA = -3.49; 	varA = 0.24;
      	mD = 0.69;  bD = -4.80; 	varD = 0.36;
  elseif ((mech == 'ss' ) | (mech == 'SS'));
      	mL = 0.62; 	bL = -2.57; 	varL = 0.15;
      	mW = 0.27; 	bW = -0.76; 	varW = 0.14;
     	mA = 0.90; 	bA = -3.42;  	varA = 0.22; 
 	mD = 0.90;  	bD = -6.32; 	varD = 0.28;
  elseif ((mech == 'ds' ) | (mech == 'DS')); 
      	mL = 0.58; 	bL = -2.42;  	varL = 0.16;
      	mW = 0.41; 	bW = -1.61; 	varW = 0.15;
     	mA = 0.98; 	bA = -3.99; 	varA = 0.26;  
      	mD = 0.08; 	bD = -0.74; 	varD = 0.38;
  end

	%%% compute mean values of length, width, area, mean slip
	Lo = mL*Mw + bL;
	Wo = mW*Mw + bW;
	Ao = mA*Mw + bA;
	Do = mD*Mw + bD;
end


%%% call function NORMSAMPLE to find values for L, W, D out
%%% of normal distributions; note that these values are the
%%% logarithms of the actual source dimensions because the
%%% scaling relations are defined based on log-log regression
if isempty(RCseed) == 1
  	RCseed = zeros(3,2);
   	[dl,RCseed(1,:)] = NormSample2(Lo,0.5*varL,100);
   	[dw,RCseed(2,:)] = NormSample2(Wo,0.5*varW,100);
   	[da,RCseed(3,:)] = NormSample2(Ao,0.5*varA,100);
   	[dd,RCseed(4,:)] = NormSample2(Do,0.5*varD,100);
elseif isempty(RCseed) ~=1 
     if isstr(RCseed) == 1;
       if RCseed == 'off'
   	dl = Lo;	
   	dw = Wo;
   	da = Ao;
   	dd = Do;
       else error('No valid RCseed-value in CalcDimfromM');
       end
     else
  	[dl,RCseed(1,:)] = NormSample2(Lo,0.5*varL,100,RCseed(1,:));
  	[dw,RCseed(2,:)] = NormSample2(Wo,0.5*varW,100,RCseed(2,:));
  	[da,RCseed(3,:)] = NormSample2(Ao,0.5*varA,100,RCseed(3,:));
  	[dd,RCseed(4,:)] = NormSample2(Do,0.5*varD,100,RCseed(3,:));
     end
end

%%% compute "final" fault length, width and mean slip;
%%% note that the results are more stable if A and W 
%%% are estimated from scaling laws, and L is computed
%% L = 10.^dl;
W = 10.^dw;
A = 10.^da;
D = 10.^dd;	if rel == 'WC'; D = 100*D; end
L = A./W;

%%% check whether the values obtained for L,W, 
%%% are compliant with fault area regression
%% A = W*L;
%% Aup = 10.^(Ao + varA); 
%% Adown = 10.^(Ao - varA);
%% if ((A > Aup) | (A < Adown))
%%	disp('  CalcCorrfromM: Condition for fault area'); 
%%	disp('  not fulfilled --> new sample is picked');	
%%	dw = NormSample(Wo,0.5*varW,100);
%%	W = 10.^dw;
%%	A = L*W;
%% end

%%% calculate mean slip [in m] for chosen Mw and 
%%% realization of fault dimensions 
if dscale == 'd'
	%%% D scaled to meet the target Mw
	Mo = fMw2MoN(Mw); 
	Mw = Mw;
	D = (Mo./(mu*L*1e3*W*1e3));	
elseif dscale == 'm'
	%%% D from distribution and 'random' Mw
	D = D/100;
	Mo = mu*L*W*1e6*D;
	Mw = fMo2MwN(Mo);
end
