function [Tr,Trc,Trm,TrS] = CalcTrfromM(Mw,stdev);
%
%  function [Tr,Trc,Trm,TrS] = CalcTrfromM(Mw,stdev) calculates
%  a rise time depending on seismic moment. The equations are
%  derived in Somerville et al. (1999) and comprise
%  a version in which the scaling is constrained to slope 1/3
%  and one in which the scaling remains unconstrained.
%
%  NOTE: these rise-times are based on slip inversions, and
%  hence are strongly biased by the particular inversion.
%  They should NEVER be used without critical judgement.
%
%  input: Mw   	- moment magnitudes
%	  stdev - give as 'y' is standard deviations are
%		  desired; those are not published, but derived
%
%  output: [Tr,Trc,Trm,TrS]- "unconstrained" and "constrained" rise time;
%		  if stdev is given as 'y', Tr, Trc will be vector
%		  containing the mean and the +/- 1sig estimates as
%		  well as a random sample of Tr, Trc (for a normal
%		  distribution
%		  Trm finally is the mean of the respective estimates
%		  Trs is computed based on Sato's (1989) empirical
%		  rise time, tr = 10^(0.5M-1.4)/80

%  Written by Martin Mai (mmai@pangea.Stanford.EDU) 
%  11/17/99
%  ------------------------------------------------

if nargin == 1; stdev = 'n'; end

Mw = Mw(:);
n = length(Mw);
Mo = fMw2Mo(Mw);	% needs moment in dyne-cm

if stdev == 'n' 

  Tr =  1.48*1e-11*(Mo.^0.42);
  Trc = 2.03*1e-9*0.33*(Mo.^0.33);
  Trm = mean([Tr'; Trc'])';

else

  %%% in natural logs
  lTr =  log(1.48*1e-11*(Mo.^0.42));
  lTrc = log(2.03*1e-9*0.33*(Mo.^0.33));

  %%% then add standard deviations to it that Nico
  %%% found to be ~40% in lognormal units
  sig = 0.4;

  slTr  = [lTr - sig;  lTr;   lTr + sig;   normrnd(lTr, 0.5*sig,n,1)]; 
  slTrc = [lTrc - sig; lTrc;  lTrc + sig;  normrnd(lTrc,0.5*sig,n,1)];

  Tr  = exp(slTr);
  Trc = exp(slTrc);
  Trm = mean([Tr'; Trc'])';

end

TrS = (1/80)*(10.^(0.5*Mw-1.4));
