function [hypo,par] = CheckHypo(S,par,hypo,search)
%
%  [hypo,par] = CheckModel(S,par,hypo,search)
%  simply checks whether the chosen/simulated hypocenter location 
%  coincides with a high-slip patch, which is usually unlikely.
%
%  INPUT: S  	 - 2D-slip distribution
% 	  par	 - parameter structure for S
%	  hypo	 - hypocenter location [x z]
%	  search - how to search for updated hypocenter
%		   'rd' for closest radially
%		   'zc' for closest vertically, 
%		   'zt' for next vertical towards the surface
%		   'zd' for next vertically in down-dip
%		   'xc' for closest horizontally
%		   'xl' for next horizontally to the left
%		   'xr' for next horizontally to the right
%		   'xz' for combination of vertical/horizontal results
%
%  OUTPUT:hypo	 - updated hypocenter location [x z]
%	  par	 - updated parameter structure for S
%
%  mmai, 03/13/2000
%  ----------------



[sz,sx] = size(S);
aspfactor = 1.5;	% factor wich defines a high-slip patch (asperity)
			% i.e. slip is larger than aspfactor*(mean slip)
			% NOTE: the smaller this factor, the larger the area
			% that is NOT allowed for the hypocenter to be located in

%%% check the hypocenter location

  %% find location of hypocenter within slip array
  W = par.dim(1);
  L = par.dim(2);
  h = par.depth - W*sin(pi*par.dip/180);      	% vertical depth to top of fault plane
  hinc = par.depth/sin(pi*par.dip/180)-W;     	% down-dip distance to top of fault plane
                                    		% parallel to fault plan
  ddc = h + par.lz;

  hx = find(par.lx >= hypo(1)-par.samp(1) & par.lx <= hypo(1)+par.samp(1));
  hz = find(ddc    >= hypo(2)-par.samp(2) & ddc    <= hypo(2)+par.samp(2));

  if length(hx) > 1; hx = hx(2); end
  if length(hz) > 1; hz = hz(2); end


  %% define areas that corresponds to the high-slip patch
  g = S >= mean(S(:))*aspfactor;

  %% see whether hypocenter falls into one of these points,
  %% if yes, move it to the closest location outside the high-slip area 

  if g(hz,hx) ~= 1   %% hypocenter outside high-slip patch
     %% nothing to do here!

  elseif g(hz,hx) == 1   %% hypocenter within some high-slip patch
     
     disp('hypocenter falls within a high-slip area, and will be');
     disp('changed slightly to a new location:');

     [i,j] = find(g ~= 1);

	%% -- Radial -- %%

     %% search for closest location radially
     if search == 'rd'	
       r = zeros(size(i));
       r = sqrt((hx-j).^2 + (hz-i).^2);
       rmin = find(r == min(r));
       Hx = j(rmin);  Hz = i(rmin);

	%% -- Vertical -- %%

     %% search for closest location vertically
     elseif search == 'zc'
       q = find(j == hx);	% extract the correct column
       dz = abs(hz-i(q));
       dzmin = find(dz == min(dz));
       Hx =hx; Hz = i(q(dzmin));

     %% towards the surface
     elseif search == 'zt'
       q = find(j == hx);	% extract the correct column
       k = find(i(q) < hz);
       Hx =hx; Hz = i(q(k(end)));

     %% in down-dip direction
     elseif search == 'zd'
       q = find(j == hx);	% extract the correct column
       k = find(i(q) > hz);
       Hx =hx; Hz = i(q(k(1)));

	%% -- Horizontal -- %%

     %% search for next location horizontally
     elseif search == 'xc'
       p = find(i == hz);	% extract the correct row
       dx = abs(hx-j(p));
       dxmin = find(dx == min(dx));
       Hz = hz;  Hx = j(p(dxmin));

     %% to the left
     elseif search == 'xl'
       p = find(i == hz);	% extract the correct row
       k = find(j(p) < hx);
       Hz = hz;  Hx = j(p(k(end)));

     %% to the right
     elseif search == 'xr'
       p = find(i == hz);	% extract the correct row
       k = find(j(p) > hx);
       Hz = hz;  Hx = j(p(k(1)));


	%% -- Combine X, Z -- %%

     %% use vertical and horizontal search results together
     elseif search == 'xz'
       q = find(j == hx);
       dz = abs(hz-i(q));
       dzmin = find(dz == min(dz));
       p = find(i == hz);
       dx = abs(hx-j(p));
       dxmin = find(dx == min(dx));
       Hx = j(p(dxmin)); Hz = i(q(dzmin));

    end      

    disp(['old hypo: x = ',num2str(hypo(1)),', z = ',num2str(hypo(2))]);
    
    hypo = [Hx(1)*par.samp(1) Hz(1)*par.samp(2)+h];
   
    disp(['NEW hypo: x = ',num2str(hypo(1)),', z = ',num2str(hypo(2))]);
end

par.hypo = hypo;
