function hypR = RandHypo(n,hypo0,depth,xzrange,xzmeth)
%
%  hypR = RandHypo(n,hypo0,depth,xzrange,xzmeth) creates a set of n 
%  randomly distributed hypocenters centered within radius R 
%  about the central input hypocenter hyp0
%
%  INPUT: n	 - number of new "randomized" hypocenters
%	  hypo0	 - central hypocenter [x z]
%	  depth	 - max depth of fault plane; cannot be exceeded
%	  xzrange- vector [dx dz] that defines the range over
%		   which the x,z-location of the hypocenter may vary
%	  xzmeth - vector ['xmeth'; 'zmeth'] to specify the distribution
%		   used to randomize the locations
%		   xmeth = 'U' or 'N' for uniform or normal distribution
%		   zmeth = 'U','N' or 'L' for uniform, normal lognormal 
%
%  OUTPUT: hypR	 - array of size [nx2] with hypocenter data
%
%  Written by Martin Mai (mmai@pangea.Stanford.EDU) 
%  05/05/98
%  last change 08/18/99
%  ------------------------------------------------


if nargin <= 3;
	xzrange = [1 1];
	xzmeth = ['N';'N'];
elseif nargin <= 4;
	xzmeth = ['N';'N'];
end

if hypo0(2) >= depth; 
	error('hypocentral depth outside fault plane');
end

xmeth = xzmeth(1);
zmeth = xzmeth(2);


%%% initialize random number generator
rand('state',sum(100*clock));


%%% randomly distributed random x locations within the given 
%%% range of allowed horizontal locations

if (xmeth == 'U' | xmeth == 'u')

	%%% uniformly distributed
	rx = unifrnd(-xzrange(1),+xzrange(1),2*n,1);

elseif (xmeth == 'N' | xmeth == 'n')

	%%% normally distributed
	rx = normrnd(0,xzrange(1),2*n,1); 
end


%%% for the vertical location we could use either a normal distribution
%%% over a small depth interval or a lognormal over the entire fault width,
%%% although the later option doesn't work that well

if (zmeth == 'U' | zmeth == 'u');

	%%% uniformly distributed
	rz = unifrnd(-xzrange(2),+xzrange(2),2*n,1);

elseif (zmeth == 'N' | zmeth == 'n')

	%%% normally distributed
	rz = normrnd(0,xzrange(2),2*n,1);

elseif (zmeth == 'L' | zmeth ==  'l');

	%%% log-normally distributed random z-location
	rz = lognrnd(0,xzrange(2),2*n,1);
	rz = abs(max(rz)-rz) - 3*std(rz);
end 


%%% "randomized" hypocentral coordinates
hx = hypo0(1) + rx;
hz = hypo0(2) + rz;

if (zmeth == 'L' | zmeth ==  'l');
	hz = hz - max(hz) + depth;
end


%%% check that "randomized" hypocentral coordinates 
%%% don't fall outside fault plane
xmin = 0.2;		
zmax = depth-0.2;	% ensure minimum distance to fault boundaries

[i,j] = find(hx > xmin);
Hx = hx(i);

[p,q] = find(hz < zmax);
Hz = hz(p);


%%% return only the requested number of hypocenters
for i = 1:n
	hypR(i,:) = [Hx(i) Hz(i)];
end


%%% round to nearest 1/10
hypR = round(hypR*10)/10;
