function [Tr,tpar] = RupTimeGrid2(dim,samp,hypo,depth,dip, ...
			vprof,vrat,dv,outfile);
%
%  [Tr,tpar] = RupTimeGrid2(dim,samp,hypo,depth,dip,vprof,vrat,dv,'outfile') 
%  calculates the rupture times for an earthquake on a planar fault given 
%  the hypocenter coordinates hypo(x,z) for fault dimensions (lx X lz),
%  velocity profile vprof and the ratio of rupture to shear wave velocity vrat
%  (typical 0.7-0.9 shear wave velocity); variations dv {in %} are allowed 
%  as well, equal for all v(i)
%
%  NOTE: This function computes the rupture times for a slip 
%  distribution defined on grid points (rather than on subfaults),
%  opposed to RUPTIME that is used for the later case. It assumes
%  that the input slip distribution is already defined on grid points.
%
%  input: dim	 - rupture dimensions [W L]
%	  samp   - sampling in z,x [dz dx]
%	  hypo	 - coordinates of hypocenter 
%		    (x along fault, z absolute depth)
%	  depth	 - max. depth of rupture plane; if depth = [], the fault
%			width is used (plus some small safety margin of 1m)
%			allowing surface rupture
%	  dip	 - dip-angle of fault plane		
%	  vprof	 - velcity-depth profile [z vp vs rho]; give 4 columns
%	  vrat	 - desired rupture to shear wave velocity ratio
%	  dv	 - random variation to rupture velocity 
%		   [dt rseed] to specify variation in secs and seed-value;
%		   if seed is omitted, it will be generated and saved
%	  outfile- optional: file name to write rupture times to
%
%  output:Tr	 - array of rupture times of each subfault 
%	  tpar	 - structural array with rupture parameters
%
%  mmai, 2/20/98
%
%  last changes:
%	05/22/2000 - allow to read parameters from structural array
%	03/14/2000 - modification of random rupture times
%  ----------------------------------------------------------------

if nargin == 8; outfile = 'n'; end

%%% read all required input from structural array, if given
if isstruct(dim) == 1
  disp('  ** read parameters from given structural array **');
  par = dim; outfile = 'n';
  dim = par.dim;	grd  = par.grd;
  samp  = par.samp; 	hypo = par.hypo;
  depth = par.depth;	dip  = par.dip;
  vprof = par.vprof;	vrat = par.vrat;
  if isfield(par,'par.ruptime') == 1
       	dv = par.ruptime.dv; TRseed = par.ruptime.TRseed;
  else
 	dv = 0; 
  end
end

x0 = hypo(1);			%% hypocenter coordinates
z0 = hypo(2);
lz = dim(1);			%% rupture dimensions	
lx = dim(2);
zsamp = samp(1);		%% given sampling
xsamp = samp(2);

%%% set-up max rupture depth if not specified; safety margin for
%%% surface rupture: 5 meters
if isempty(depth) == 1 | depth == lz;
	depth = lz+0.005; 
end

%%% set up vectors of along strike/down dip dimensions
%%% note that the length of dx, dz is increased by one, and
%%% hence is based on a node-grid definition
h = depth - lz*sin(pi*dip/180);
ztop = (z0-h)/sin(pi*dip/180);
zbot = (depth-z0)/sin(pi*dip/180);
dx = [-x0:xsamp:lx-x0];
dz = [-ztop:zsamp:zbot];

%%% calculate radial distance from hypocenter
[x,z] = meshgrid(dx,dz);
r = sqrt((z).^2 + (x).^2);

%%% interpolate S-velocity profile onto grid-points
if length(vprof) ~= 1
  zz = vprof(:,1);
  vv = vprof(:,3);
  zfault = [h:zsamp*sin(pi*dip/180):depth]';  % depth points on fault plane
  vs = interp1(zz,vv',zfault); 	% vs-profile on fault plane
  vr = vs*vrat;			% profile of rupture velocity
else
  disp('vs constant over ENTIRE fault');
  vs = vprof;
  vr = vs*vrat;
  vr = ones(size(r,1),1)*vr;
end

%%% unperturbed rupture time
V = ones(size(r));
for i = 1:size(r,2)
   V(:,i) = vr;
end
t = r./V;

%%% add random perturbation to rupture time, if desired,
%%% variations follow a uniform distribution in [-dv dv]
if dv ~= 0
     if length(dv) == 1
	TRseed = sum(100*clock);
     else
	TRseed = dv(2);
     end
     dt = dv(1)*mean(samp);
     rand('seed',TRseed);
     %dT = -dt + 2*dt*rand(size(t));
     dT = unifrnd(-dt,dt,size(t));
else
     dT = 0;
     TRseed = [];
end

%%% final rupture time history, adjusted for grid points, i.e.
%%% shifted such that Tr = 0 at hypocenter; 
Tr = t+dT;

%tmin = min(T(:));
%Tr = T;
%Tr = T - tmin;

%%% write rupture times to output file if requested
if (outfile ~= 'n')
  disp('  Rupture timees will be written to file:');
  disp(['  -->  ',outfile,'  <--']);
  WriteArray(Tr,[W L],outfile);
end

%%% assemple final structure for TRseed
tpar.vr = vr;
if dv ~= 0
  tpar.TRseed = TRseed;
  tpar.dv = dv;
  tpar.model = 'r';
else
  tpar.TRseed =[];
  tpar.dv = [];
  tpar.model = '';
end
