function [S,par] = SlipReal4(srcpar,mech,acf,corr,seed,samp,grd,...
			nexp,wlevel,taper,depth,dip,outfile,fig)
%
%  [S,par] = SlipReal4(srcpar,'mech','acf',corr,seed,samp,grd, ...
%			nexp,wlevel,taper,depth,dip,outfile,fig) 
%
%  simulates a dislocation model for given source parameters and 
%  source mechanism. If not given, source dimensions can be 
%  computed from empirical scaling relations.
%  The slip on the fault surface is calculated using the spectral 
%  synthesis method in which the slip-amplitude spectrum is defined 
%  through a spatial auto-correlation function or a power law decay;
%  the phase is random in U(-pi,pi), but the entire field obeys
%  Hermitian symmetry.
%  The variables SRCPAR and MECH are required input; for all other 
%  input parameters, an empty array [] will select the default values
%  (see below).
%
%  INPUT:
%  srcpar - array-structure with ALL source parameters OR 
%	    cell-array with source parameters and string to identify scaling;
%	    OR simple vector with source parameters
%	    for the last two options, srcpar can be of the form
%		{Mw 'rel'} --  source dimensions computed from scaling laws
%	               rel == 'MB' uses Mai & Beroza (2000)
%		       rel == 'WC' uses Wells & Coppersmith (1994)
%		       rel == 'WG' uses USGS WorkingGroup99 M vs. A (2000)
%		{A 'rel'}  --  area A (in km^2), rel is 'MB' or 'WC'
% 		[W L]      --  Mw estimated from Wells & Coppersmith (1994),
%		[W L Mw]   --  D computed from given parameters
%	     	[W L Mw D] --  Mw will be scaled to match given D!
%
%           NOTE: fault width W and length L are needed in km, mean slip D in m
%
%	    In case srcpar is given as an array-structure, make sure the naming
%	    in your array is EXAKT the same as needed in the code; the best idea
%	    is to run SlipReal4 once with a standard set of input values, and then
%	    modify the entries in the output-structure PAR.
%
%  mech   - faulting mechanism for the simulated event, needed to
%		    compute source parameters from scaling relations
%		    'ss' or 'SS' for strike-slipevents;  
%	     	'ds' or 'DS' for dip-slip events
%	     	'al' or 'AL' if both types should be considered
%			(may be useful in case of normal faulting events)
%
%  acf    - string to denote autocorrelation function
%	    	'ak' or 'AK' for anisotropic von Karman ACF
%	     	'ex' or 'EX' for exponential ACF
%	     	'fr' or 'FR' for the fractal case (power law decay)
%	     	'gs' or 'GS' for Gaussian ACF
%		    for this option YOU have to SPECIFY the correlation length
%		    if [], default 'ak' is used
%
%  corr   - correlation length and/or spectral decay parameters
%	     	[az ax] for Gauss or exponential ACF
%	     	[az ax H] for von Karman ACF H = Hurst number)
%	     	[D kc] for fractal slip where D is the fractal dimension;
%		    kc: corner wavenumber beyond which the spectrum decays;
%		    kc is related to the source dimensions, and is computed
%		    as kc = 2*pi/(sqrt(L*W)) if it is not given
%	     	[] if corr is an empty matrix, the relevant parameters for the
%		    given autocorrelation function will be computed
%		    (NOT true for the Gaussian since no relations have been
%		    established between corr length and source parameters)
%
%  seed   - structural array of seed values for the random-number generator, 
%	     	called at various locations in the code; if seed == [], then 
%	     	the code uses an expression like 
%			'Rseed = sum(100*clock);' 
%			'randn('seed', Rseed);' <-- uses MATLAB4 generators!!
%	     	to generate the random numbers; the value Rseed is stored in
%	     	the output structure par.
%	     	The sequence is as follows, and is the same that is returned
%	     	by the code:
%			seed.SS = SSseed; 1x2-array, used in SpecSyn2
%			seed.WL = WLseed; 1x1-array, used in WaterLevel
%			seed.CS = CSseed; 2x2-array, used in CalcSigfromD
%			seed.RC = RCseed; 3x2-array, used in CalcDimfromM
%			seed.RWC = RWCseed; 3x2-array, used in CalcDimWC
%			seed.CR = CRseed; 1x1, 4x2, 5x2, used in CalcCorrfromLW
%	     	Hence, you can run the code once, get the corresponding array 
%	     	and use it again to create the EXACT SAME random slip model.
%
%  samp   - sampling of dislocation model in z,x direction [dz dx]
% 	     	NOTE: the final sampling may be finer in case the option 'pow2' 
%		    is given as 'y' (see SPECSYN2) or sampling must be adjusted for
%		    the source dimensions
%
%  grd	  - slip function to be defined on grid-nodes or subfaults 
%	     	'nod' for grid-node definition [grid-size (L/dx+1)*(W/dz+1)] 
%	     	'sub' for sub-fault definition [grid-size (L/dx) * (W/dz)] ; [dz is downdip increment]
%
%  nexp   - non-linear scaling exponent for the random field (i.e S = S^nexp) 
%	     	nexp < 1 smoothens the fields (steepens the spectrum) 
%	     	nexp == 1 doesn't change anything (default)
%	     	nexp > 1 roughens the field (flattens the spectrum)
%	     	the purpose of this variable is to allow for simulation of
%	     	slip distributions with large peak-slip values;
%	     	a good choice for that is usually nexp = 1.25 - 1.5;
%
%  wlevel - method to scale the zero-mean random field to nonnegative slip
%		    wlevel == []: field "lifted" above zero (default) 
%	     	wlevel ==  0: values < 0 will be set equal to zero 
%	       	wlevel == -1: values < 0 will be set to small random value
%	     	wlevel == -2: values < 0 will be set to 0.25*field value
%	     	wlevel == -3: values < mean-slip set to small random value
%	     	wlevel == scalar: values < scalar set to small random value
%	     	NOTE: wlevel = [] preserves the spectral decay; wlevel = -2
%	     	also preserves the slip spectrum rather well without intro-
%	     	ducing too much artificial high-wavenumbers, yet scales the
%	     	slip such that locally large-slip patches are present
%
%  taper  - tapering the edges of the slip model 
%	     	[left/right top bottom] (in km) for customized tapering
%		    'y' for default tapering of 2.5 km (i.e [2.5 2.5 2.5])
%	     	[left/right top bottom P] where P is an exponent smaller 1,
%	    	applies an additional 'depth-taper' of the form z^P 
%	        (i.e.some kind of sqrt-function) to mimick less slip in the
%	     	upper crustal regions close to the surface
%
%  depth  - max. depth of rupture plane;
%           option with depth range [zmin zmax] not implemented
%		    zmin is the depth to the top of the fault plane			  
%		    zmax is the maximum depth of fault plane (in km)  
%			 (default: zmin = 0, zmax =15)  
%
%  dip	  - dip angle of fault plane (in degrees) (default = 90 deg)  
%
%  outfile- optional: string for a filename to which the code writes 
%	     	the slip realization as a simple ascii 2D-array, where rows
%	     	correspond to depth (default: 'n')
%
%  fig	  - optional: 'y' to view the slip realization; this will open
%	     	a figure window for each realization (set to 'n' if called 
%	     	in a loop) (default: 'y')
%		   
%  OUTPUT:	
%  S 	  - 2D-slip distribution
%  par 	  - structure with all relevant source parameters 
%
%  Type 'type SlipReal4' to view the entire program and particularly the
%  more detailed description of the program at the beginning.

%
%
%  Last changes: 
%
%	05/17/2000 - srcpar can now also be a array-stucture in which all
%		relevant source parameters are given; this makes it easier
%		to re-create models or to modify only one of the source
%		parameters, but keep everything else as is.
%	01/03/2000 - extended WaterLevel function; included option to
%		scale field non-linearly; possibility to pre-specify
%		seed-values for random number generators
%	11/27/1999 - ability to simulate dislocation distributions with 
%		arbitrary grid spacing

%	=====================================================================
%
%	CHANGES (03/01/2000)
%
%	The WaterLevel-function is extended such that not it now also may
%	take the option of using the actual down-scaled field values 
%	below the given threshold to fill the "zeroed" values. Currently
%	the scaling factor is hard-wired to be 0.25. This somewhat better
%	preserves the spectral shape and hence the spatial structure of the
%	models.
%	I also included the possibility to scale the field by some exponent,
%	NEXP. NEXP > 1 roughens the fields (flattens the spectrum), NEXP < 1
%	smoothens the field (steepens the spectrum), NEXP = 1 leaves it as is.
%	The philosophy behind this variable is to be able to generate slip 
%	models with large peak slip values in an otherwise "normally"
%	heterogeneous field.
%	The output structure PAR now also stores the seed-values used
%	in the simulations. The reason being that we can re-use the
%	same exact seed-values at the corresponding locations in the code
%	to re-create the exact same model, although the simulations are 
%	essentially random! This is possible because the random number 
%	generators are not entirely "random", they are pseudo-random number 
%	generators that always create the same numbers for a given seed. 		
%
%	CHANGES (11/27/1999)
%
%	Implemented the option to specify sampling interval and hence
%	create dislocation distributions with arbitrary grid spacing in
%	both directions. The major change actually involves the function
%	SpecSyn2 which computes the random field.
%
%	CHANGES (11/08/1999)
%
%	- modified function CalcSigfromD to reflect new regression results
%	- changed scaling of the simulated random field with slip variation;
%	  here, a water level can be introduced (wlevel == 1) yielding
%	  patchier models, but the level (i.e. all values < 0) is fixed 
%
%	IMPORTANT NOTES (Oct. 25, 1999)
%
%	*) The current implementation of the spectral synthesis method 
%	(see SpecSynAn) to simulate spatial random fields assumes that the 
%	grid-size is 1 km in both directions. Hence, the random field is 
%	constructed on grid nodes with dx = dz = 1km. In most applications, 
%	finite-source slip models with spatial sampling of 1 km provide 
%	accurate enough realizations for near-source strong motion simulation 
%	up to frequencies of engineering interest. 
%	A future version will be able to simulate random fields with finer 
%	discretization.
%	In case you are happy with coarser discretization, you can always 
%	downsample any particular realization using the function interpgrid. 
%	
%	*) Currently, the program computes slip realizations defined on 
%	subfaults, each of size 1x1km (see note above). If you want to compute 
%	slip realizations on grid-nodes, you can easily do so by changing a 
%	few lines just before and after the call to function SpecSynAn.
%
%	*) Note that the data that went into developing this program were very
%	limited, i.e we analyzed about 30 finite-source slip models, in a 
%	magnitude range of 5.9 - 8.1. When performing the regression to obtain 
%	scaling parameters we did not impose any positivity constraints, and 
%	hence you may encounter 'negative' correlation length for small 
%	arthquakes (M < 5.5). It would be therefore saver to specify the 
%	correlation length and the source dimensions for earthquakes smaller, 
%	say M = 6.0.
%
%	*) As of Oct 1999, the computation of correlation lengths from 
%	empirical results (function CalcCorrfromLW) is implemented only for 
%	the specification of strike-slip or dip-slip events, and also only 
%	handles the cases for exponential and anisotropic von Karman 
%	autocorrelation as well as the fractal case. The rationale behind this 
%	is that in most application you will know the likely mechanism of the 
%	earthquakes you are interested in. Also, since we found that the
%	Gaussian and isotropic von Karman (options 'gs' and 'ik') are not well 
%	suited to model heterogeneous slip I haven't yet included it into the
%	modeling scheme. I will do so as soon as possible, simply for the sake 
%	of completeness.
%	
%	*) Introducing a waterlevel to create "patchier" slip distributions 
%	does not quite work as intented (see function WaterLevel). Setting 
%	variable wlevel to 0 or 0.1 does have the desired effect, but a future
%	version may have a better implementation.
%
%
%	Written by Martin Mai (mmai@pangea.Stanford.EDU) 
%	02/19/99
%	last change 10/25/1999
%	================================================


%%% ----------------
%%% FIXED PARAMETERS
%%% ----------------

mu = 3.3*1e10;		% rigidity of crustal rocks
pow2 = 'n';		% used to force to FFT-size for the random field
			% to be a power of two; if desired, set to 'y'
tapwin = 'hn';		% taper window used; either 'kw' for Kaiser window 
			% (steep) or 'hn' for smoother Hanning window

synnp2 = 'n';		% set to 'y' if fields to be simulated on square-grids
			% of power-of-two size, and then resampled to desired
			% grid-size


%%% -----------------------------------------
%%% SET AND DISPLAY DEFAULT VALUES, DEPENDING
%%% ON WHAT IS GIVEN BY THE FUNCTION CALL
%%% -----------------------------------------

if nargin == 1
   if isstruct(srcpar) == 1
	disp('  Reading source parameters from given structural array');
	outfile = 'n';		fig = 'n';
        mech = srcpar.mech;	acf  = srcpar.acf;
	corr = srcpar.corr;	seed = srcpar.Rseed;
        samp = srcpar.samp;	nexp = srcpar.nexp;
	grd = srcpar.grd;
  	depth = srcpar.depth;	dip  = srcpar.dip;
	wlevel = srcpar.wlevel; taper = srcpar.taper;
	src = srcpar; 		srcpar = zeros(1,4);
	srcpar(1) = src.dim(1); srcpar(2) = src.dim(2);
	srcpar(3) = src.Mw;	srcpar(4) = src.D/100;	
   else
	disp('  Need structural array as input or at least');
	disp('  moment magnitude Mw and source mechanism'); 
	error('  **** SlipReal4 stopped ****');
   end
elseif nargin == 2
	acf = 'ak'; 	corr = []; 	seed = [];
	samp = [1 1];	grd = 'nod';	nexp = 1;	
	wlevel = []; 	taper = 'y'; 	depth = 15; 	
	dip = 90;	outfile = 'n'; 	fig = 'y';  
	disp('  Default values selected in SlipReal:'); 	
	disp('  ACF = AK; 	SAMP  = [1 1];	GRD = nod;	NEXP = 1;'); 	
 	disp('  WLEVEL = [];	TAPER = y;	DIP = 90 dg.; 	DEPTH  = 15 km');
elseif nargin == 3	
	corr = []; 	seed = [];	samp = [1 1];
	grd = 'nod';	nexp = 1;	wlevel = []; 
	taper = 'y'; 	depth = 15; 	dip = 90;
	outfile = 'n'; 	fig = 'y';  
	disp('  Corr. length: simulated');
	disp('  Default values selected in SlipReal:'); 	
	disp('  SAMP  = [1 1];	GRD = nod;	NEXP = 1;  	WLEVEL = []');
	disp('  TAPER = y;	DIP = 90 dg.; 	DEPTH  = 15 km');
elseif nargin == 4	
	seed = []; 	samp = [1 1];	grd = 'nod';	nexp = 1;	
	wlevel = []; 	taper = 'y'; 	depth = 15; 	dip = 90;
	outfile = 'n'; 	fig = 'y';  
	disp('  Default values selected in SlipReal:'); 	
	disp('  SAMP  = [1 1];	GRD = nod;	NEXP = 1;  	WLEVEL = []');
	disp('  TAPER = y;	DIP = 90 dg.; 	DEPTH  = 15 km');
elseif nargin == 5; 
	samp = [1 1];	grd = 'nod';	nexp = 1;	
	wlevel = []; 	taper = 'y'; 	depth = 15; 	dip = 90;
	outfile = 'n'; 	fig = 'y';
	disp('  Default values selected in SlipReal:'); 	
	disp('  SAMP  = [1 1];	GRD = nod;	NEXP = 1;  	WLEVEL = []');
	disp('  TAPER = y;	DIP = 90 dg.; 	DEPTH  = 15 km');
elseif nargin == 6; 
	grd = 'nod';	nexp = 1;	wlevel = []; 	
	taper = 'y'; 	depth = 15; 	dip = 90;
	outfile = 'n'; 	fig = 'y';
	disp('  Default values selected in SlipReal:'); 	
	disp('  GRD = nod;	NEXP = 1;  	WLEVEL = []');
	disp('  TAPER = y;	DIP = 90 dg.; 	DEPTH  = 15 km'); 
elseif nargin == 7; 
	nexp = 1;	wlevel = []; 	taper = 'y'; 	
	depth = 15; 	dip = 90;
	outfile = 'n'; 	fig= 'y';
	disp('  SAMP and NEXP used as given');
	disp('  Default values selected in SlipReal:'); 	
	disp('  --> NEXP = 1; WLEVEL = []; TAPER = y; DIP = 90 dg.; DEPTH  = 15 km');
elseif nargin == 8; 
	wlevel = [];	taper = 'y'; 	depth = 15; 	
	dip = 90;	outfile = 'n'; 	fig= 'y';
	disp('  SAMP, NEXP, WLEVEL used as given');
	disp('  Default values selected in SlipReal:'); 	
	disp('  --> WLEVEL = []; TAPER = y; DIP = 90 dg.; DEPTH  = 15 km');
elseif nargin == 9;
	taper = 'y';	depth = 15; 	dip = 90;
	outfile = 'n'; 	fig= 'y';
	disp('  SAMP, NEXP, WLEVEL, used as given');
	disp('  Default values selected in SlipReal:'); 	
	disp('  --> TAPER = y; DIP = 90 dg.; 	DEPTH  = 15 km');
elseif nargin == 10; 
	depth = 15;	dip = 90;	outfile = 'n'; 	fig= 'y';
	disp('  SAMP, NEXP, WLEVEL, TAPER used as given');
	disp('  Default values selected in SlipReal:'); 	
	disp('  --> DEPTH = 15 km, DIP = 90 dg.');
elseif nargin == 11; 
	dip = 90;	outfile = 'n'; 	fig= 'y';
	disp('  SAMP, NEXP, WLEVEL, TAPER, DEPTH used as given');
	disp('  Default values selected in SlipReal:'); 	
	disp('  --> DIP = 90 dg.');
elseif nargin == 12, outfile = 'n'; 	fig= 'y';
elseif nargin == 13, fig = 'y';
end


%%% ----------------------------------
%%% SOME SIMPLE CHECKS ON INPUT ERRORS
%%% ----------------------------------

%%% handle the case where acf is given and the corr length
%%% vector, but the length of corr doesn't comply with the
%%% given function
if isempty(srcpar) == 1;
	disp('----');
	disp('Come on ... - give me at least some source parameter ...');
	disp('----');
	error('  **** SlipReal4 stopped ****');
end

if ((acf == 'ex') | (acf == 'EX') | (acf == 'gs') | (acf == 'GS'))
    if (length(corr) == 1) | (length(corr) > 2)
	disp('  -- Number of elements given in vector CORR');
	disp('     does not comply with given acf-function');
	error('  **** SlipReal stopped ****');
    end
elseif ((acf == 'ak') | (acf == 'AK')) 
    if (length(corr) == 1) | (length(corr) == 2) | (length(corr) > 3)
	disp('  -- Number of elements given in vector CORR');
	disp('     does not comply with given acf-function');
	error('  **** SlipReal stopped ****');
    end
elseif ((acf == 'fr') | (acf == 'FR'))
    if (length(corr) >= 3)
	disp('  -- Number of elements given in vector CORR');
	disp('     does not comply with given acf-function');
	error('  **** SlipReal stopped ****');
    elseif (length(corr)) == 1
	disp('  -- corner wavenumber kc not specified');
	disp('     will be computed from source dimensions');
    end
end



%%% -----------------------------------------------------------------
%%% SET RANDOM SEED ARRAYS
%%% the random seed numbers are stored in arrays to be able to later
%%% recreate the EXACT same slip realization; 
%%% set the initial seed values to empty arrays and have the 
%%% subroutines return the actual seed values into the structural 
%%% sub-array par.Rseed; OR
%%% retrieve seed-values from given structural array
%%% -----------------------------------------------------------------
if isempty(seed) == 1   	
    RCseed = [];		 
    RWCseed = [];		
    CRseed = [];
    SSseed = [];
    CSseed = [];
    WLseed = [];
    disp('  new SEED-values used');
else
    SSseed = seed.SS; 		
    WLseed = seed.WL;	
    CSseed = seed.CS;   
    RCseed = seed.RC;	
    RWCseed = seed.RWC;	
    CRseed = seed.CR;
    disp('  SEED-values from previous simulation used');
end


%%% -----------------------------------------------------------------
%%% NON-LINEAR SCALING EXPONENT FOR SLIP
%%% the purpose of this variable, together with the 'water level' 
%%% option, is to be able to "customize" the generated models, e.g. 
%%% create large peak slip values;
%%% nexp > 1 roughens the field (flatter spectrum)
%%% nexp < 1 smoothens the field (steeper spectrum)
%%% we generally run nexp = 1, but test showed nexp = 1.2-1.5 is a 
%%% reasonable good choice
%%% ------------------------------------
if isempty(nexp) == 1	
  nexp = 1;			 
end			
	

%%% ----------------------
%%% GET RUPTURE DIMENSIONS
%%% ----------------------
%%% computed from empirical scaling relations 
%%% or specified by the user, depending on option
%%% 'MB or 'WC' to use Mai & Beroza (2000)
%%% scaling relations or Wells & Coppersmith (1994)

if iscell(srcpar) == 1

   rel = srcpar{2};
   par = srcpar{1}; 
   
   
   %%% distinguish cases of A or Mw based on value of par
   %%% because there are no earthquakes with M > 10 !
   if length(par) > 1
      disp('')
      disp('Give either area A or magnitude M');
      disp('along with string for scaling relations');
      error('SLIP REAL TERMINATED');
   else
      if par < 10
	   Mw = par;
	   if rel == 'WG'; 
		disp('WorkingGroup Relations only for given area!');
		error('SLIP REAL TERMINATED');
	   end
	  [L,W,D,Mw,RCseed] = CalcDimfromM(Mw,mech,'d',rel,RCseed);
       else
	   A = par;
	  [L,W,D,Mw,RCseed] = CalcDimfromA(A,mech,rel,RCseed);
       end
   end
	
      if rel == 'MB'
	   disp('  Source dimensions from Mai & Beroza (2000)');
	   par.srcmeth = 'empirical (M&B)';
      elseif rel == 'WC'
	   disp('  Source dimensions from Wells & Coppersmith (1994)');
   	   par.srcmeth = 'empirical (W&C)';
      end

   %%% srcpar is NOT given as cell array     
   elseif iscell(srcpar) ~= 1

      %%% if W,L are given, only Wells & Coppersmith (1994) relations are useful
      if length(srcpar) == 2
	W = srcpar(1);
	L = srcpar(2);
	[L,W,D,Mw,RWCseed] = CalcDimDMWC(L,W,mech,RWCseed);
	par.srcmeth = 'empirical (W&C)';
	disp('  Moment Magnitude Mw from Wells & Coppersmith (1994)');


     %%% handle cases in which fault length, width, mean slip and
     %%% moment magnitude are given
     elseif length(srcpar) == 3
	W = srcpar(1);
	L = srcpar(2);
	Mw = srcpar(3);
	Mo = fMw2MoN(Mw); 
	D = Mo/(mu*L*W*1e6);
	par.srcmeth = 'specified';
	disp('  Mean Slip D computed from given source parameters');

     elseif length(srcpar) == 4
	W = srcpar(1);
	L = srcpar(2);
	Mw = srcpar(3);
	D = srcpar(4);
	Mo = mu*(L*W*D*1e6);
	Mw = fMo2MwN(Mo);
	par.srcmeth = 'specified';
	disp('  Moment Magnitude Mw adjusted to reflect given Mean Slip D');

     end
  
end

%%% check that maximum fault width does not exceed maximum allowable depth; 
%%% if that's case, the width will be adjusted to max allowable width
maxW = depth/sin(pi*dip/180);
if W > maxW
   disp('  -- computed fault width exceeds max. allowable width');
   disp('     width will be adjusted to max. allowable width');
   W = floor(maxW);
end

%%% round values for simulations, and adjust mean slip accordingly
L = round(L);		
W = round(W);
Mo = fMw2MoN(Mw);
D = Mo./(L*W*1e6*mu);

%%% print final source parameters to screen
s1 = sprintf('  ** Final Source Parameters: \n');
s2 = sprintf('     L = %.1f km, W = %.1f km, D = %.2f m\n',L,W,D);
s3 = sprintf('     Mw = %.2f, Mo = %.3e Nm',Mw,Mo); 
disp([s1,s2,s3]);


%%% -----------------------------------------------------------------
%%% TAPERING OF SLIP FUNCTION, [left/right top bot wfct]
%%% to avoid large slip at the fault boundaries, we apply a taper 
%%% function that may vary for the left/right, top and bottom extent
%%% of the taper window; the default is set to 2.5 km. Additionally,
%%% a depth-dependent "weighting function" can be applied such as to 
%%% have larger slip values at depth
%%% -----------------------------------------------------------------
if iscell(taper) == 1 
 if size(taper,2) == 3;
    twfr  = taper{3};
    twdpt = taper{2};
    taper = taper{1};
 elseif size(taper,2) == 2;
    twdpt = taper{2};
    taper = taper{1};
 else
    taper = taper{1};
 end
elseif length(taper) == 4;
  twdpt = taper(4);
elseif isempty(taper) == 1;
  taper = [0 0 0];		% array of zeros in case of NO taper
end
if taper == 'y'		    	
  twkm = [2.5 2.5 2.5];	    	% set taper windows to default values 
elseif taper == 'd'
  if exist('twfr','var') == 0;
     twfr = 0.25;
  end
  twL = twfr*L; twW = twfr*W;	% set taper window length to 25% of 
  twkm = [twL twW twW];		% the source size in each direction
elseif length(taper)>=3	    	
  twkm = taper(1:3);		% set taper windows to given values	    	
end

if length(samp) == 2, 
	ntx = round(twkm(1)/samp(2)); 
	ntt = round(twkm(2)/samp(1));
	ntb = round(twkm(3)/samp(1));	 
elseif length(samp) == 1, 
	ntx = round(twkm(1)/samp(1)); 
	ntt = round(twkm(2)/samp(1));
	ntb = round(twkm(3)/samp(1));
end

tapslp = [ntx ntt ntb]; 	% defines the # of rows/cols of the output slip 	
				% distribution to be tapered



%%% -----------------------------------------------
%%% PARAMETERS FOR AUTOCORRELATION FUNCTION OF SLIP
%%% -----------------------------------------------

%%% call function to compute correlation length/spectral decay for given Mw
%%% if vector with correlation length is empty
if isempty(corr) == 1
	[corr,CRseed] = CalcCorrfromLW([W L Mw],mech,acf,CRseed);
	par.corrmeth = 'simulated';
	disp('  Spectral decay parameters computed from empirical relations');
  %% check that the aspect ratio of the correlation length has the same "sign"
  %% as the fault dimensions
	if acf == 'ak' | acf == 'ex' | acf == 'gs'
           if W > L & corr(1) < corr(2); 
		corr(1:2) = [corr(2) corr(1)]; 
 	   end
	end

elseif (length(corr)) == 1 & (acf == 'fr' | acf == 'FR') 
	%%% need to compute corner wavenumber, based on source dimensions
	kc = 2*pi/(sqrt(L*W));
	corr = [corr(1) kc];
else
	par.corrmeth = 'specified';
	disp('  Spectral decay parameters used as specified');
end

%%% handle case in which the given or computed Hurst number falls outside
%%% the allowed range, 0 < H < 1, and correct those invalid values 
%%% in case they were computed; otherwise error message and terminate
if ((acf == 'ak') | (acf == 'AK'))
   H = corr(3);
   if H < 0.5  
	disp('  ++ Hurst exponent H < 0.5 theoretically NOT allowed ++');
   end
   if H >= 1,
	corr(3) = 0.99;
	disp('  -- Hurstnumber computed: H > 1'); 
	disp('     corrected to max. allowed H = 0.99'); 
   elseif H <= 0; 
	corr(3) = 0.01;
	disp('  -- Hurstnumber computed: H < 0');
	disp('     corrected to min. allowed H = 0.01');
	disp('     NOTE: spectrum will be very flat, ');
	disp('     generating a very heterogeneous slip model');
   end
end


%%% print final spectral decay parameters to screen
d1 = sprintf('  Final Decay Parameters: \n');
if ((acf == 'ex') | (acf == 'EX') | (acf == 'gs') | (acf == 'GS'))
  d2 = sprintf('      az = %.2f km, ax = %.2f km',corr); 
elseif ((acf == 'ak') | (acf == 'AK'))
  d2 = sprintf('      az = %.2f km, ax = %.2f km, H = %.2f',corr);
elseif ((acf == 'fr') | (acf == 'FR'))
  d2 = sprintf('      D = %.2f, kc = %.2f',corr);
end
disp([d1,d2]);



%%% ---------------------------------
%%% GENERATE THE SPATIAL RANDOM FIELD
%%% ---------------------------------
%%% call spectral synthesis function to create random field
%%% we use SpecSyn2 which allows to specify arbitrary sampling
%%% intervals in either direction
 
if synnp2 == 'y'
   disp('   --->  use next power of two grid size for simulation');
   nx = L/samp(2); nz = W/samp(1);
   nmax = max([nx nz]);
   ns = 2.^nextpow2(nmax);		%% new grid-size
   xsc = ns/L;     zsc = ns/W;
  
   %%% now adjust correlation length to reflect the new size
   if acf == 'ak'
      ncorr = [corr(1)*zsc corr(2)*xsc corr(3)];
   elseif acf == 'ex' | acf == 'gs';
      ncorr = [corr(1)*zsc corr(2)*xsc];
   end

   %%% simulate field, down-sample (interpolate) onto
   %%% desired grid size, and set parameters correctly
   [s,spar] = SpecSyn2([ns ns],[1 1],ncorr,acf,SSseed);
    s1 = resample(s,nz,ns);
    G = resample(s1',nx,ns)';
   spar.synmeth = 'synnp'; spar.synN = ns;
   spar.dim = [W L]; 	 spar.samp = samp;
   spar.lx = [0:samp(2):(L-1)*samp(2)];
   spar.lz = [0:samp(1):(W-1)*samp(1)];
   spar.corr = corr;  	 spar.size = size(G); 
    
else

  %% this is the standard option to use
  [G,spar] = SpecSyn2([W L],samp,corr,acf,SSseed);

end
SSseed = spar.Rseed;		% save the seed value
G = G - mean(G(:));		% to ensure that the simulated random field
G = G/std2(G);			% has zero mean and unit variance


%%% crude check that positive values are concentrated in the interior
%%% of the field; this should avoid large slip at the boundaries which
%%% will result in large stresses and unrealistically "patchy" slip models
[lz,lx] = size(G);   % lz is downdip coordinate
px = round(lx/sqrt(2));		% dimensions for 'interior' area
pz = round(lz/sqrt(2));	
qx = floor(0.5*(lx-px));	% indices for 'interior' area
qz = floor(0.5*(lz-pz));		
GI = G(qz+1:end-qz,qx+1:end-qx);
if mean(GI(:)) < mean(G(:)), G = -1*G; end


%%% ---------------------------------------------------------
%%% RESIZE AND SCALE FIELD TO MATCH DESIRED SOURCE PARAMETERS
%%% ---------------------------------------------------------

%%% resize grid (by bilinear interpolation) in case of subfault definition
%%% Note that spectral synthesis tool works on grid-dimensions of ODD size!
if grd == 'sub'
  disp('  Slip defined on SUBFAULTS selected');
  [G,spar.lx,spar.lz] = interpgrid(G,spar.samp,spar.dim,spar.samp);
else
  disp('  Slip defined on GRID-NODES selected'); 
end


%%% perform a non-linear transformation, if desired, to create models
%%% with higher peak-slip values
if nexp ~= 1
	disp(['  Non-linear scaling of slip function: S=S^',num2str(nexp)]);
	G = G - min(G(:));
	G = G.^nexp;
	G = G - mean(G(:));
end


%%% calculating mean slip variation dsig and max displacement dmax
%%% from empirical data; needs input D in cm;
[dsig,dmax,CSseed] = CalcSigfromD(100*D,mech,CSseed);


%%% scale zero-mean, unit-variance random field by slip variation sigma 
%%% using twice the computed mean slip as allowed standard deviation
%%% D is assumed to be in meters, here we need D in cm!
X = D*100*(1 + 2*G);		


%%% --------------------------------------------------------
%%% MAKE SLIP POSTIVE DEFINITE, AND APPLY TAPER TO THE EDGES
%%% --------------------------------------------------------

%%% two possibilities to make field positive-definite:
%%% 1) set all values < 0 (or any other cut-off value)
%%%    equal zero which effectively changes the spectral decay 
%%% 2) add min. value, i.e. "lift" field above zero level
%%% the first method yields patchier models with moment close to the
%%% desired one, the second preserves the heterogeneity spectrum 
if isempty(wlevel) == 1
	disp('  Water-level: field simply ''lifted'' above zero');
	%%% "lift" field above zero level
	Y = X - min(X(:));
	WLseed = [];
elseif isempty(wlevel) ~= 1
    if wlevel == 0
	disp('  Water-level: values < 0 set to zero');
	%%% set values smaller 0 equal zero, 
	Y = X - min(X(:));
	[Y,WLseed] = WaterLevel(Y,'abs',abs(min(X(:))),'n',[]);
    elseif wlevel == -1
	%%% set values smaller 0 equal zero, with small random
	%%% values of 0.05*max(field) added
	disp('  Water-level: values < 0 set to small random value');
	Y = X - min(X(:));
	[Y,WLseed] = WaterLevel(Y,'abs',abs(min(X(:))),'y',WLseed);
    elseif wlevel == -2
	%%% set values smaller 0 equal zero, with 0.25*field-value
	%%% in place of the zeros
	disp('  Water-level: values < 0 set to 0.25 their original value');
	Y = X - min(X(:));
	[Y,WLseed] = WaterLevel(Y,'abs',abs(min(X(:))),'d',[]);
    elseif wlevel == -3
	%%% use the computed mean slip (in cm) as cut-off
	disp('  Water-level: values < D set to small random values');
	Y = X - min(X(:));
	[Y,WLseed] = WaterLevel(Y,'abs',100*D,'y',WLseed);
    else
	%%% use user-specified value as cut-off
	disp(['  Water-level: values <',num2str(wlevel), ...
		' set to small random values']);
	Y = X - min(X(:));
	[Y,WLseed] = WaterLevel(Y,'abs',wlevel,'y',WLseed);
    end
end


%%% taper slip at edges to obtain zero slip amplitudes at fault boundaries
%%% the taperwidth is specified in a vector of the form [left/right top bottom]
%%% Kaiser-window ('kw') yields very sharp edges, Hanning-window ('hn') is 
%%% less steep
%%% if the given taper vector has 4 elements, the last one (P) is used to 
%%% apply simple "depth-taper" by scaling the field as S^P in down-dip
%%% direction. P = 0.1-0.5 is a reasonable choice

%if W >= 0.95*depth
%  disp('  ----------  SURFACE RUPTURE ---------- ');
%  disp('  rupture W >= 0.95*depth, taper adjusted'); 
%  disp('  ---------------------------------------');  
%  tapslp = [tapslp(1) 0 tapslp(3)];
%end

if taper == 'y' | taper == 'd' | length(taper) >= 3

  disp('  Slip function tapered at the boundaries:');
  tapstr = sprintf('    left/right, top, bottom: [%.1f %.1f %.1f]', twkm);
  disp([tapstr,' km']);

  T = TaperSlip(Y,tapslp,tapwin);

  if exist('twdpt','var') == 1
    disp(['    Additional depth taper applied: Sz^',num2str(twdpt)]);
    w = [linspace(1,size(T,1)*samp(1),size(T,1))'].^twdpt;
    w = w./max(w);
    for k = 1:size(T,2), T(:,k) = T(:,k).*w; end
  end

elseif isempty(taper) == 1
  disp('  Slip function not tapered at the boundaries');
  T = Y;
end


%%% make sure that field is indeed positive definite, with
%%% smallest allowed slip value in all subfaults being zero
T = T - min(T(:));

%%% rescale slip distribution to match desired/computed moment
[mo,mw] = fmomentN(T,[W L]);
Mo = fMw2MoN(Mw);
S = T*Mo/mo;
d = mean(S(:));


%%% -------------------------------------
%%% CREATE FINAL OUTPUT ARRAYS/STRUCTURES
%%% -------------------------------------

%%% assemble output structure with all relevant source parameters
par.mech = mech;
par.dim = spar.dim;
par.samp = spar.samp;
par.lx = spar.lx;
par.lz = spar.lz;  %% "lz is downdip coordinate"
%par.pow2 = spar.pow2;
par.grd = grd;
if isfield(spar,'synmeth') == 1;
  par.synmeth = spar.synmeth;
  par.synN; spar.synN;
end
par.D = d;
par.dmax = max(S(:));
par.dmaxreg = dmax;
par.Mo = Mo;
par.Mw = Mw;
par.acf = acf;
par.corr = corr;
par.nexp = nexp;
par.wlevel = wlevel;
par.taper = twkm;
if exist('twdpt','var') == 1
  par.taper = [twkm twdpt];
end
par.depth = depth;
par.dip = dip;
par.Rseed.SS = SSseed; 
par.Rseed.WL = WLseed;
par.Rseed.CS = CSseed;
par.Rseed.RC = RCseed;
par.Rseed.RWC = RWCseed;
par.Rseed.CR = CRseed;


%%% ------------------------
%%% DISK or GRAPHICAL OUTPUT
%%% ------------------------
%%% create output, if requested: write to disk
%%% or plot to screen

%%% write slip realization to output file if requested
if (outfile ~= 'n')
  disp('  Slip realization will be written to file:');
  disp(['  -->  ',outfile,'  <--']);
  WriteArray(S,[W L],samp,outfile);
end

% %%% simple plot of resulting slip realization
% if fig == 'y' 
%    disp('  Slip realization will be graphically displayed');
%    plotslip(S,[W L],samp,[-1 -1],depth,'hor','y');
% end
