function [hypo_x,hypo_z] = stat_hypo(par,mech)
%---------------------------------------------------------
% Script to calculate probabilistic hypocenter positions
% basing upon the paper of Mai et al.(2005).
%
% INPUT
%        slip - slip values on the fault plane
%        step - step (in Km) of the slip grid
%        grid - type of grid representation for the input slip
%               model
%        mech - faulting mechanism ['dp' -> general dip-slip]
%                                  ['dc' -> crustal dip-slip]
%                                  ['ds' -> subduction dip-slip]
%                                  ['ss' -> strike-slip]
%                                  ['al' -> global]
%        Mw   - moment magnitude 
%
% OUTPUT
%        [hypo_x,hypo_z] - hypocenter coordinates on the fault
%                          plane in km
%
% *****
% This script allows the user to interactively select one or more
% reasonable hypocenters. Probabilities for location on the entire 
% plane are depicted (normalized to 1), accordingly to the
% relationships described in Mai et al.(2005).
%
% Author: Walter Imperatori, June 2008
%---------------------------------------------------------

flag='n';  %temporary flag for other constraints

% fault grid-dimensions
dim=size(par.slip);
dim_z=dim(1);
dim_x=dim(2);

step= par.samp(1);   % !samp assumed to be isotropic !

grid= par.grd;
Mw = par.Mw;

slip= par.slip;

%if (grid == 'sub')

   
 
   % fault physical dimensions (in km)
%   strike_len=(dim_x)*step;
%   dip_len=(dim_z)*step;

   % creates plot axis (in km)
%   x=[step/2:step:(strike_len-step/2)];
%   y=[step/2:step:(dip_len-step/2)];

%elseif (grid == 'nod')

   % fault physical dimensions (in km)
%   strike_len=(dim_x-1)*step;
%   dip_len=(dim_z-1)*step;

   % creates plot axis (in km)
%   x=[0:step:strike_len];
%   y=[0:step:dip_len];
%end

%-------------------------------------------------------
% computes pdf for on-plane position

% --> downdip
if (mech == 'dp')
   hypz=gampdf(linspace(0,1,dim_z),7.364,0.072);
elseif (mech == 'dc')
   hypz=wblpdf(linspace(0,1,dim_z),0.692,3.394);
elseif (mech == 'ds')
   hypz=gampdf(linspace(0,1,dim_z),12.658,0.034);
elseif (mech == 'ss')
   hypz=wblpdf(linspace(0,1,dim_z),0.626,3.921);
elseif (mech == 'al')
   hypz=wblpdf(linspace(0,1,dim_z),0.612,3.353);
end

% --> along strike
hypx=normpdf(linspace(0,1,dim_x),0.5,0.23);

% calculates and scales to 1 probability based on on-plane position
hypz=hypz';
hyp_pos=hypz*hypx;
hyp_pos=hyp_pos/max(max(hyp_pos));

%------------------------------------------------------
% computes pdf based on maximum and mean slip ratio

% --> computes maximum and mean slip values
[notzero_z,notzero_x]=find(slip > 0.0);
for i=1:(max(size(notzero_z)))             %exctracts where slip is not zero
    notzero_slip(i)=slip(notzero_z(i),notzero_x(i));
end
mean_slip=mean(notzero_slip);
max_slip=max(max(slip));

% --> computes pdf for mean slip ratio
tmp_slip=slip/mean_slip;
if(mech == 'dp' | mech == 'dc' | mech == 'ds')
   hyp_mean=gampdf(tmp_slip,2.616,0.623);
elseif (mech == 'ss')
   hyp_mean=gampdf(tmp_slip,1.928,0.868);
elseif (mech == 'al')
   hyp_mean=gampdf(tmp_slip,2.210,0.748);
end

% --> computes pdf for maximum slip ratio
tmp_slip=slip/max_slip;
if(mech == 'dp' | mech == 'dc' | mech == 'ds')
   hyp_max=wblpdf(tmp_slip,0.454,1.874);
elseif (mech == 'ss')
   hyp_max=wblpdf(tmp_slip,0.446,1.551);
elseif (mech == 'al')
   hyp_max=wblpdf(tmp_slip,0.450,1.688);
end

% --> computes probability based on both previous slip ratios and scales to 1
hyp_slip=hyp_max.*hyp_mean;
hyp_slip=hyp_slip/max(max(hyp_slip));

% -------------------------------------------------------
% computes probability based on both on-plane position and slip ratios
slip_pos=hyp_slip.*hyp_pos;  
slip_pos=slip_pos/max(max(slip_pos));   % scales it to 1 

% -------------------------------------------------------
% define asperities regions (definitions from Mai et al. 2005)
[out_asp_dip,out_asp_strike]=find(slip < 0.33*max_slip);
[asp_dip,asp_strike]=find(0.33*max_slip <= slip & slip < 0.66*max_slip);
[big_asp_dip,big_asp_strike]=find(slip >= 0.66*max_slip);

% defines asperities positions on the fault plane (0=no asperity, 1=asperity, 2=big asperity)
asp_fault(1:dim_z,1:dim_x)=0;
for i=1:(max(size(asp_dip)))
    asp_fault(asp_dip(i),asp_strike(i))=1;
end
for i=1:(max(size(big_asp_dip)))
    asp_fault(big_asp_dip(i),big_asp_strike(i))=2;
end

% -------------------------------------------------------------------------------------------
% applies other constraints (paragraph "Distance between Hypocenter and Closest Asperities" of
% Mai et al. (2005)) --------> to be improved!



% --> refreshes hypocenter locations probability according to position inside/outside asperities
for i=1:(max(size(out_asp_dip)))
    slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.48;
%    mask(out_asp_dip(i),out_asp_strike(i))=mask(out_asp_dip(i),out_asp_strike(i))*0.48;
end
for i=1:(max(size(asp_dip)))
    slip_pos(asp_dip(i),asp_strike(i))=slip_pos(asp_dip(i),asp_strike(i))*0.35;
%    mask(asp_dip(i),asp_strike(i))=mask(asp_dip(i),asp_strike(i))*0.35;
end
for i=1:(max(size(big_asp_dip)))
    slip_pos(big_asp_dip(i),big_asp_strike(i))=slip_pos(big_asp_dip(i),big_asp_strike(i))*0.16;
%    mask(big_asp_dip(i),big_asp_strike(i))=mask(big_asp_dip(i),big_asp_strike(i))*0.16;
end

slip_pos=slip_pos/max(max(slip_pos));   % scales to 1


 if(flag == 'y')

% --> calculates R_max in Km (on the entire fault plane, also where slip=0)
for i=1:dim_z
   for j=1:dim_x
      for k=1:(max(size(notzero_z)))
         euclid(k)=sqrt(((notzero_z(k)-1)*step-(i-1)*step)^2 + ((notzero_x(k)-1)*step-(j-1)*step)^2);
      end
      R_max(i,j)=max(euclid);
   end
end

% --> defines peak slip and distances to it in Km (on the entire fault plane, also where slip=0)
[tmp_peak_z,tmp_peak_x]=find(slip == max(max(slip)));
peak_z=tmp_peak_z(1);   %takes first maximum if there are more than one
peak_x=tmp_peak_x(1);
for i=1:dim_z
   for j=1:dim_x
      peak_dist(i,j)=sqrt(((peak_z-1)*step-(i-1)*step)^2 + ((peak_x-1)*step-(j-1)*step)^2);
   end
end

% --> calculates shortest distance to asperities in Km (only for points outside them)
for i=1:size(out_asp_dip)
   for j=1:size(asp_dip)
      eul(j)=sqrt(((out_asp_dip(i)-1)*step-(asp_dip(j)-1)*step)^2 + ((out_asp_strike(i)-1)*step-(asp_strike(j)-1)*step)^2);
   end
   asp_dist(out_asp_dip(i),out_asp_strike(i))=min(eul);
end

% --> calculates shortest distance to big asperities in Km (only for points outside them)
for i=1:size(out_asp_dip)
   for j=1:size(big_asp_dip)
      eul2(j)=sqrt(((out_asp_dip(i)-1)*step-(big_asp_dip(j)-1)*step)^2 + ((out_asp_strike(i)-1)*step-(big_asp_strike(j)-1)*step)^2);
   end
   big_asp_dist(out_asp_dip(i),out_asp_strike(i))=min(eul2);
end
for i=1:size(asp_dip)
   for j=1:size(big_asp_dip)
      eul3(j)=sqrt(((asp_dip(i)-1)*step-(big_asp_dip(j)-1)*step)^2 + ((asp_strike(i)-1)*step-(big_asp_strike(j)-1)*step)^2);
   end
   big_asp_dist(asp_dip(i),asp_strike(i))=min(eul3);
end

mask(1:dim_z,1:dim_x)=1;   %initialize mask

% --> applies constraint valid for all hypocenters (on the entire fault) respect to R_max
[tap_z,tap_x]=find(peak_dist./R_max < 0.04 | peak_dist./R_max > 0.8 );

for i=1:size(tap_z)
%   slip_pos(tap_z(i),tap_x(i))=0.0;
    mask(tap_z(i),tap_x(i))=0.0;
end

% --> constraint for hypocenters outside asperities
for i=1:size(out_asp_dip)
   if (asp_dist(out_asp_dip(i),out_asp_strike(i)) <= 0.15*R_max(out_asp_dip(i),out_asp_strike(i)))
%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.95;
      mask(out_asp_dip(i),out_asp_strike(i))=mask(out_asp_dip(i),out_asp_strike(i))*0.95;
   end
   if ((asp_dist(out_asp_dip(i),out_asp_strike(i)) > 0.15*R_max(out_asp_dip(i),out_asp_strike(i))) & (asp_dist(out_asp_dip(i),out_asp_strike(i)) <= 0.3*R_max(out_asp_dip(i),out_asp_strike(i))))
%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.05;
      mask(out_asp_dip(i),out_asp_strike(i))=mask(out_asp_dip(i),out_asp_strike(i))*0.05;
   end
   if (asp_dist(out_asp_dip(i),out_asp_strike(i)) > 0.3*R_max(out_asp_dip(i),out_asp_strike(i)))
%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.0;
      mask(out_asp_dip(i),out_asp_strike(i))=mask(out_asp_dip(i),out_asp_strike(i))*0.0;
   end
end

% --> constraint for hypocenters outside big asperities
for i=1:size(out_asp_dip)
   if (big_asp_dist(out_asp_dip(i),out_asp_strike(i)) <= 0.4*R_max(out_asp_dip(i),out_asp_strike(i)))
%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.95;
      mask(out_asp_dip(i),out_asp_strike(i))=mask(out_asp_dip(i),out_asp_strike(i))*0.95;
   end
   if ((big_asp_dist(out_asp_dip(i),out_asp_strike(i)) > 0.4*R_max(out_asp_dip(i),out_asp_strike(i))) & (big_asp_dist(out_asp_dip(i),out_asp_strike(i)) <= 0.6*R_max(out_asp_dip(i),out_asp_strike(i))))
%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.05;
      mask(out_asp_dip(i),out_asp_strike(i))=mask(out_asp_dip(i),out_asp_strike(i))*0.05;
   end
   if (big_asp_dist(out_asp_dip(i),out_asp_strike(i)) > 0.6*R_max(out_asp_dip(i),out_asp_strike(i)))
%      slip_pos(out_asp_dip(i),out_asp_strike(i))=slip_pos(out_asp_dip(i),out_asp_strike(i))*0.0;
      mask(out_asp_dip(i),out_asp_strike(i))=mask(out_asp_dip(i),out_asp_strike(i))*0.0;
   end
end 
for i=1:size(asp_dip)
   if (big_asp_dist(asp_dip(i),asp_strike(i)) <= 0.4*R_max(asp_dip(i),asp_strike(i)))
%      slip_pos(asp_dip(i),asp_strike(i))=slip_pos(asp_dip(i),asp_strike(i))*0.95;
      mask(asp_dip(i),asp_strike(i))=mask(asp_dip(i),asp_strike(i))*0.95;
   end
   if ((big_asp_dist(asp_dip(i),asp_strike(i)) > 0.4*R_max(asp_dip(i),asp_strike(i))) & (big_asp_dist(asp_dip(i),asp_strike(i)) <= 0.6*R_max(asp_dip(i),asp_strike(i))))
%      slip_pos(asp_dip(i),asp_strike(i))=slip_pos(asp_dip(i),asp_strike(i))*0.05;
      mask(asp_dip(i),asp_strike(i))=mask(asp_dip(i),asp_strike(i))*0.05;
   end
   if (big_asp_dist(asp_dip(i),asp_strike(i)) > 0.6*R_max(asp_dip(i),asp_strike(i))) 
%      slip_pos(asp_dip(i),asp_strike(i))=slip_pos(asp_dip(i),asp_strike(i))*0.0;
      mask(asp_dip(i),asp_strike(i))=mask(asp_dip(i),asp_strike(i))*0.0;
   end
end 

mask=mask/max(max(mask));
%figure;contour(x,y,mask,[0.2 0.4 0.6 0.8 1],'k')
%figure;imagesc(x,y,mask)

% applies mask
slip_pos=slip_pos.*mask;

%normalize to 1
slip_pos=slip_pos/max(max(slip_pos));

end

x=par.lx;
y=par.lz;

% plot figures
%figure;imagesc(x,y,peak_dist);figure,imagesc(x,y,R_max);
figure; subplot(3,1,1), imagesc(x,y,hyp_pos), colorbar('horiz');
%subplot(2,3,2), imagesc(x,y,hyp_mean), colorbar('horiz'); subplot(2,3,3), imagesc(x,y,hyp_max), colorbar('horiz');
subplot(3,1,2), imagesc(x,y,hyp_slip),colorbar('horiz');
subplot(3,1,3), imagesc(x,y,slip_pos),colorbar('horiz'), hold on, contour(x,y,asp_fault,[1 2],'w');
%subplot(2,3,6),imagesc(x,y,slip_pos),colorbar('horiz');hold on;plot(hypo_x,hypo_z,'p','MarkerSize',10,'MarkerFaceColor','w')

figure;
subplot(2,1,1),imagesc(x,y,slip),colorbar('horiz');
subplot(2,1,2),imagesc(x,y,slip_pos),colorbar('horiz');



find reasonable hypocenters interactively
[hypo_x,hypo_z]=getpts;

subplot(2,1,2),imagesc(x,y,slip_pos),colorbar('horiz'); hold on; plot(hypo_x,hypo_z,'p','MarkerSize',12,'MarkerFaceColor','w')
subplot(2,1,1),imagesc(x,y,slip),colorbar('horiz'); hold on; plot(hypo_x,hypo_z,'p','MarkerSize',12,'MarkerFaceColor','w')





% sort out probability values in a descend fashion
%max_prob=sort(max(slip_pos),'descend');

%size_hyp=(dim_z*dim_x);    %total vectorial dimensions

% ----------------------------------------------------------------------------------------
% different methods to find out reasonable hypocenter locations
%--- first method: highest absolute value
%count=0; flag=0;
%for k=1:size_hyp
%  [tmp_z,tmp_x]=find(slip_pos == max_prob(k));
%  for i=1:size(tmp_z)       %loop effectively performed only if more points have the same value
%     if (peak_dist(tmp_z(i),tmp_x(i)) > 0.04*R_max(tmp_z(i),tmp_x(i)))   %condition valid for all models
%        count=count+1;
%        flag=1;
%        tmp2_z(count)=tmp_z(i);
%        tmp2_x(count)=tmp_x(i);
%     end
%  end
%  if (flag == 1) 
%     break
%  end
%end
% if there are more candidates, one of them is randomly chosen
%if (count > 1) then
%   i=ceil(unifrnd(0,count));
%end
%hypo_x=(tmp2_x(i)-1)*step; hypo_z=(tmp2_z(i)-1)*step;
%--- second method: random selection above a given threshold 

%for i=1:size_hyp
%   [cand_z,cand_x]=find(slip_pos,max_prob(i));
%end
% ---------------------------------------------------------------------------------------------