function [Y,WLseed] = WaterLevel(G,meth,wl,rfil,WLseed)
%
%  function [Y,WLseed] = WaterLevel(G,'meth',wl,'rfil',WLseed) 
%  introduces a waterlevel in the function, i.e sets values
%  smaller than a given value equal to zero.
%  Default cut-off is 10% of the maximum value in G
%
%  INPUT:
%  G 	  - 1D- or 2D-array
%  meth	  - 'per' for wl as percentage of max. value in G
%	    'abs' for an absolute value
%  wl	  - water level (default = 0.1*max(G))		
%  rfil	  - option to fill-in zeros with small random numbers
%	    (rfil == 'y'), rfil == 'n' for zeros or rfil == 'd' 
%	    to have down-scaled values from the model 
%  WLseed - seed value for random number generator
%
%  OUTPUT:
%  Y	  - 1D- or 2D-array, same size as input array
%  WLseed - 'seed' for the random numbers (empty array if unused)

%
%  Written by Martin Mai (mmai@pangea.Stanford.EDU) 
%  08/21/99
%  ------------------------------------------------
	
if nargin == 1;
	meth = 'per'; wl = 0.10; rfil = 'n'; WLseed = [];
elseif nargin == 2; 
	wl = 0.10; rfil = 'n'; WLseed = []
elseif nargin == 3;
	rfil = 'n'; WLseed = [];
elseif nargin == 4;
	WLseed = [];
end

%%% introduce water-level 
if meth == 'per'
	g = G > wl*max(G(:));
elseif meth == 'abs'
	g = G > wl;
end

%%% introduces random small non-zeros entries where we had zeros before
if rfil == 'y'		
	Y = (G.*g);
	d = g == 0;
        if isempty(WLseed) == 1 
	   WLseed = sum(100*clock);
	end	
	randn('seed', WLseed);
	r = randn(size(d));
	r = 0.05*max(Y(:))*(d.*r);
	Y = Y + r;	

%%% this option uses the actual values that are below the threshold and
%%% sets them to 0.25 their value
elseif rfil == 'd'
	g(g==0) = 0.25;
	Y = G.*g;
	WLseed = [];
else
	Y = (G.*g);
	WLseed = [];
end
