function [OUT]=extend_fault(IN,np,tfun,fig)
%
% [OUT]=extend_fault(IN,np,tfun,fig)
%
% IN:   2D array to be extended
% np:   number of points added at each side
%       default: np=5
% tfun: string specifying the type of tapering
%       {'linear'} | 'hanning' | 'kaiser' |
%       'none'
% fig:  flag for figure output (optional)
%       {'fig'} | 'nofig'
%
% Surround a 2D array (e.g. slip on a fault plane)
% by a zone of np points width. In this zone the values
% drop to zero, except for tfun='none'.
%
% J.Ripperger, 15.09.2003

% 23.09.2003: added tfun='none'
%
% Last change: 08.10.2003: added optional flag 'fig' 

if nargin<2
  np=5;
end

if nargin<3 | isempty(tfun)
  tfun='linear';
end

if nargin<4 | isempty(fig)
  fig='fig';
end

nyi=size(IN,1);
nxi=size(IN,2);

nxo=np+nxi+np;
nyo=np+nyi+np;

% Initialize output array
OUT=zeros(nyo,nxo);

% Copy input array to center of output array
OUT(np+1:nyo-np,np+1:nxo-np)=IN;

% Create taper
switch tfun
case 'linear'
  taper=linspace(0,1,np+1);

case 'hanning'
  hn=hanning(2*np+1)';
  taper=zeros(1,np+1);
  taper(1:np+1)=hn(1:np+1);
  
case 'kaiser'
  beta=6;
  taper=zeros(1,np+1);
  ka=kaiser(2*np+1,beta);
  taper(1:np+1)=ka(1:np+1);

case 'none'
  taper=ones(1,np+1); 

otherwise
  fprintf(1,'Invalid specification of tfun!');
  return
end

% Taper in x direction
OUT(:,1:np)=OUT(:,np+1)*taper(1:np);
OUT(:,nxo-np+1:nxo)=OUT(:,np+nxi)*fliplr(taper(1:np));

% Taper in y direction
OUT(1:np,:)=taper(1:np)'*OUT(np+1,:);
OUT(nyo-np+1:nyo,:)=flipud(taper(1:np)')*OUT(np+nyi,:);

% Create figure only if corresponding flag is set
if isequal(fig,'fig')
  figure
  subplot(2,1,1)
  imagesc(IN);
  axis equal
  set(gca,'Xlim',[1 nxo])
  set(gca,'Ylim',[1 nyo])

  subplot(2,1,2)
  imagesc(OUT);
  axis equal;
  set(gca,'Xlim',[1 nxo])
  set(gca,'Ylim',[1 nyo])
end

return

